@@ -14,6 +14,7 @@
 
 
 class VarTree;
+class ExprValue;
 class ExprWnd;
 class KDebugger;
 class QStrList;
@@ -91,6 +92,7 @@ enum DbgCommand {
 	DCcondition,
 	DCsetpc,
 	DCignore,
+	DCprintWChar,
 	DCsetvariable
 };
 
@@ -205,7 +207,7 @@ struct FrameInfo
 struct StackFrame : FrameInfo
 {
     int frameNo;
-    VarTree* var;			/* more information if non-zero */
+    ExprValue* var;			/* more information if non-zero */
     StackFrame() : var(0) { }
     ~StackFrame();
 };
@@ -376,7 +378,7 @@ public:
     /**
      * Parses the output as an array of QChars.
      */
-    virtual VarTree* parseQCharArray(const char* output, bool wantErrorValue, bool qt3like) = 0;
+    virtual ExprValue* parseQCharArray(const char* output, bool wantErrorValue, bool qt3like) = 0;
 
     /**
      * Parses a back-trace (the output of the DCbt command).
@@ -434,23 +436,19 @@ public:
      * @param newVars Receives the parsed variable values. The values are
      * simply append()ed to the supplied list.
      */
-    virtual void parseLocals(const char* output, QList<VarTree>& newVars) = 0;
+    virtual void parseLocals(const char* output, QList<ExprValue>& newVars) = 0;
 
     /**
      * Parses the output of a DCprint or DCprintStruct command.
      * @param output The output of the debugger.
      * @param wantErrorValue Specifies whether the error message should be
      * provided as the value of a NKplain variable. If this is false,
-     * #var will be 0 if the printed value is an error message.
-     * @param var Returns the variable value. It is set to 0 if there was
-     * a parse error or if the output is an error message and wantErrorValue
-     * is false. If it is not 0, #var->text() will return junk and must be
-     * set using #var->setText().
-     * @return false if the output is an error message. Even if true is
-     * returned, #var might still be 0 (due to a parse error).
+     * 0 is returned if the printed value is an error message.
+     * @return the parsed value. It is 0 if there was a parse error
+     * or if the output is an error message and #wantErrorValue
+     * is \c false. The returned object's text() is undefined.
      */
-    virtual bool parsePrintExpr(const char* output, bool wantErrorValue,
-				VarTree*& var) = 0;
+    virtual ExprValue* parsePrintExpr(const char* output, bool wantErrorValue) = 0;
 
     /**
      * Parses the output of the DCcd command.
