@@ -54,7 +54,7 @@ DebuggerMainWnd::DebuggerMainWnd(const char* name) :
     m_btWindow = new QListBox(dw1, "backtrace");
     dw1->setWidget(m_btWindow);
     KDockWidget* dw2 = createDockWidget("Locals", p, 0, i18n("Locals"));
-    m_localVariables = new ExprWnd(dw2, "locals");
+    m_localVariables = new ExprWnd(dw2, i18n("Variable"), "locals");
     dw2->setWidget(m_localVariables);
     KDockWidget* dw3 = createDockWidget("Watches", p, 0, i18n("Watches"));
     m_watches = new WatchWindow(dw3, "watches");
@@ -84,6 +84,7 @@ DebuggerMainWnd::DebuggerMainWnd(const char* name) :
 
     connect(m_watches, SIGNAL(addWatch()), SLOT(slotAddWatch()));
     connect(m_watches, SIGNAL(deleteWatch()), m_debugger, SLOT(slotDeleteWatch()));
+    connect(m_watches, SIGNAL(textDropped(const QString&)), SLOT(slotAddWatch(const QString&)));
 
     KAction* windowMenu = actionCollection()->action("window");
     m_filesWindow->setWindowMenu(static_cast<KActionMenu*>(windowMenu)->popupMenu());
@@ -118,15 +119,6 @@ DebuggerMainWnd::DebuggerMainWnd(const char* name) :
     // tab width
     connect(this, SIGNAL(setTabWidth(int)), m_filesWindow, SIGNAL(setTabWidth(int)));
 
-    // Establish communication when right clicked on file window.
-    connect(m_filesWindow, SIGNAL(filesRightClick(const QPoint &)),
-	    SLOT(slotFileWndMenu(const QPoint &)));
-
-    // Connection when right clicked on file window before any file is
-    // loaded.
-    connect(m_filesWindow, SIGNAL(clickedRight(const QPoint &)),
-	    SLOT(slotFileWndEmptyMenu(const QPoint &)));
-
     // file/line updates
     connect(m_filesWindow, SIGNAL(fileChanged()), SLOT(slotFileChanged()));
     connect(m_filesWindow, SIGNAL(lineChanged()), SLOT(slotLineChanged()));
@@ -158,8 +150,8 @@ DebuggerMainWnd::DebuggerMainWnd(const char* name) :
     connect(dockManager, SIGNAL(change()), SLOT(updateUI()));
 
     // popup menu of the local variables window
-    connect(m_localVariables, SIGNAL(rightPressed(int, const QPoint&)),
-	    this, SLOT(slotLocalsPopup(int, const QPoint&)));
+    connect(m_localVariables, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)),
+	    this, SLOT(slotLocalsPopup(QListViewItem*, const QPoint&)));
 
     restoreSettings(kapp->config());
 
@@ -381,6 +373,7 @@ void DebuggerMainWnd::readProperties(KConfig* config)
 
 const char WindowGroup[] = "Windows";
 const char RecentExecutables[] = "RecentExecutables";
+const char LastSession[] = "LastSession";
 
 void DebuggerMainWnd::saveSettings(KConfig* config)
 {
@@ -391,6 +384,10 @@ void DebuggerMainWnd::saveSettings(KConfig* config)
 
     m_recentExecAction->saveEntries(config, RecentExecutables);
 
+    KConfigGroupSaver g2(config, LastSession);
+    config->writeEntry("Width0Locals", m_localVariables->columnWidth(0));
+    config->writeEntry("Width0Watches", m_watches->columnWidth(0));
+
     DebuggerMainWndBase::saveSettings(config);
 }
 
@@ -414,7 +411,7 @@ void DebuggerMainWnd::restoreSettings(KConfig* config)
 	{ i18n("Output"), m_ttyWindow },
 	{ i18n("Memory"), m_memoryWindow }
     };
-    for (int i = 0; i < sizeof(dw)/sizeof(dw[0]); i++)
+    for (int i = 0; i < int(sizeof(dw)/sizeof(dw[0])); i++)
     {
 	KDockWidget* w = dockParent(dw[i].w);
 	w->setTabPageLabel(dw[i].text);
@@ -425,6 +422,15 @@ void DebuggerMainWnd::restoreSettings(KConfig* config)
 
     m_recentExecAction->loadEntries(config, RecentExecutables);
 
+    KConfigGroupSaver g2(config, LastSession);
+    int w;
+    w = config->readNumEntry("Width0Locals", -1);
+    if (w >= 0 && w < 30000)
+	m_localVariables->setColumnWidth(0, w);
+    w = config->readNumEntry("Width0Watches", -1);
+    if (w >= 0 && w < 30000)
+	m_watches->setColumnWidth(0, w);
+
     DebuggerMainWndBase::restoreSettings(config);
 
     emit setTabWidth(m_tabWidth);
@@ -514,6 +520,13 @@ void DebuggerMainWnd::slotAddWatch()
     }
 }
 
+void DebuggerMainWnd::slotAddWatch(const QString& text)
+{
+    if (m_debugger != 0) {
+	m_debugger->addWatch(text);
+    }
+}
+
 void DebuggerMainWnd::slotFileChanged()
 {
     // set caption
@@ -753,7 +766,7 @@ QString DebuggerMainWnd::makeSourceFilter()
 /*
  * Pop up the context menu in the locals window
  */
-void DebuggerMainWnd::slotLocalsPopup(int, const QPoint& pt)
+void DebuggerMainWnd::slotLocalsPopup(QListViewItem*, const QPoint& pt)
 {
     QPopupMenu* popup =
 	static_cast<QPopupMenu*>(factory()->container("popup_locals", this));
@@ -763,7 +776,7 @@ void DebuggerMainWnd::slotLocalsPopup(int, const QPoint& pt)
     if (popup->isVisible()) {
 	popup->hide();
     } else {
-	popup->popup(m_localVariables->mapToGlobal(pt));
+	popup->popup(pt);
     }
 }
 
@@ -772,10 +785,10 @@ void DebuggerMainWnd::slotLocalsPopup(int, const QPoint& pt)
  */
 void DebuggerMainWnd::slotLocalsToWatch()
 {
-    int idx = m_localVariables->currentItem();
+    VarTree* item = m_localVariables->selectedItem();
 
-    if (idx >= 0 && m_debugger != 0) {
-	QString text = m_localVariables->exprStringAt(idx);
+    if (item != 0 && m_debugger != 0) {
+	QString text = item->computeExpr();
 	m_debugger->addWatch(text);
     }
 }
@@ -802,48 +815,13 @@ void DebuggerMainWnd::slotEditValue()
 	return;				/* don't edit twice */
     }
     
-    int idx = wnd->currentItem();
-    if (idx >= 0 && m_debugger != 0 && m_debugger->canSingleStep())
+    VarTree* expr = wnd->currentItem();
+    if (expr != 0 && m_debugger != 0 && m_debugger->canSingleStep())
     {
 	TRACE("edit value");
 	// determine the text to edit
-	VarTree* expr = static_cast<VarTree*>(wnd->itemAt(idx));
 	QString text = m_debugger->driver()->editableValue(expr);
-	wnd->editValue(idx, text);
-    }
-}
-
-// Pop up the context menu of the files window (for loaded files)
-void DebuggerMainWnd::slotFileWndMenu(const QPoint& pos)
-{
-    QPopupMenu* popup =
-	static_cast<QPopupMenu*>(factory()->container("popup_files", this));
-    if (popup == 0) {
-        return;
-    }
-    if (popup->isVisible()) {
-	popup->hide();
-    } else {
-	// pos is still in widget coordinates of the sender
-	const QWidget* w = static_cast<const QWidget*>(sender());
-	popup->popup(w->mapToGlobal(pos));
-    }
-}
-
-// Pop up the context menu of the files window (while no file is loaded)
-void DebuggerMainWnd::slotFileWndEmptyMenu(const QPoint& pos)
-{
-    QPopupMenu* popup =
-	static_cast<QPopupMenu*>(factory()->container("popup_files_empty", this));
-    if (popup == 0) {
-        return;
-    }
-    if (popup->isVisible()) {
-        popup->hide();
-    } else {
-	// pos is still in widget coordinates of the sender
-	const QWidget* w = static_cast<const QWidget*>(sender());
-	popup->popup(w->mapToGlobal(pos));
+	wnd->editValue(expr, text);
     }
 }
 
@@ -943,6 +921,9 @@ void DebuggerMainWnd::slotExecAttach()
 {
 #ifdef PS_COMMAND
     ProcAttachPS dlg(this);
+    // seed filter with executable name
+    QFileInfo fi = m_debugger->executable();
+    dlg.filterEdit->setText(fi.fileName());
 #else
     ProcAttach dlg(this);
     dlg.setText(m_debugger->attachedPid());
