@@ -87,6 +87,7 @@ public slots:
     void slotFileChanged();
     void slotLineChanged();
     void slotAddWatch();
+    void slotAddWatch(const QString& text);
     void slotNewFileLoaded();
     void slotNewStatusMsg();
     void slotDebuggerStarting();
@@ -96,7 +97,7 @@ public slots:
     void slotProgramStopped();
     void slotBackTimer();
     void slotRecentExec(const KURL& url);
-    void slotLocalsPopup(int item, const QPoint& pt);
+    void slotLocalsPopup(QListViewItem*, const QPoint& pt);
     void slotLocalsToWatch();
     void slotEditValue();
 
@@ -113,9 +114,6 @@ public slots:
     void slotExecArgs();
     void intoBackground();
     void slotConfigureKeys();
-
-    void slotFileWndMenu(const QPoint& pos);
-    void slotFileWndEmptyMenu(const QPoint& pos);
 };
 
 #endif // DBGMAINWND_H
