@@ -48,14 +48,14 @@ KDebugger::KDebugger(QWidget* parent,
     m_envVars.setAutoDelete(true);
     m_brkpts.setAutoDelete(true);
 
-    connect(&m_localVariables, SIGNAL(expanding(KTreeViewItem*,bool&)),
-	    SLOT(slotLocalsExpanding(KTreeViewItem*,bool&)));
-    connect(&m_watchVariables, SIGNAL(expanding(KTreeViewItem*,bool&)),
-	    SLOT(slotWatchExpanding(KTreeViewItem*,bool&)));
-    connect(&m_localVariables, SIGNAL(editValueCommitted(int, const QString&)),
-	    SLOT(slotValueEdited(int, const QString&)));
-    connect(&m_watchVariables, SIGNAL(editValueCommitted(int, const QString&)),
-	    SLOT(slotValueEdited(int, const QString&)));
+    connect(&m_localVariables, SIGNAL(expanded(QListViewItem*)),
+	    SLOT(slotExpanding(QListViewItem*)));
+    connect(&m_watchVariables, SIGNAL(expanded(QListViewItem*)),
+	    SLOT(slotExpanding(QListViewItem*)));
+    connect(&m_localVariables, SIGNAL(editValueCommitted(VarTree*, const QString&)),
+	    SLOT(slotValueEdited(VarTree*, const QString&)));
+    connect(&m_watchVariables, SIGNAL(editValueCommitted(VarTree*, const QString&)),
+	    SLOT(slotValueEdited(VarTree*, const QString&)));
 
     connect(&m_btWindow, SIGNAL(highlighted(int)), SLOT(gotoFrame(int)));
 
@@ -723,9 +723,9 @@ void KDebugger::saveProgramSettings()
     m_programConfig->deleteGroup(WatchGroup);
     // then start a new group
     m_programConfig->setGroup(WatchGroup);
-    KTreeViewItem* item = m_watchVariables.itemAt(0);
+    VarTree* item = m_watchVariables.firstChild();
     int watchNum = 0;
-    for (; item != 0; item = item->getSibling(), ++watchNum) {
+    for (; item != 0; item = item->nextSibling(), ++watchNum) {
 	varName.sprintf(ExprFmt, watchNum);
 	m_programConfig->writeEntry(varName, item->getText());
     }
@@ -1092,6 +1092,7 @@ void KDebugger::parse(CmdQueueItem* cmd, const char* output)
 	break;
     case DCprintStruct:
     case DCprintQStringStruct:
+    case DCprintWChar:
 	handlePrintStruct(cmd, output);
 	break;
     case DCinfosharedlib:
@@ -1210,9 +1211,9 @@ void KDebugger::updateAllExprs()
     }
 
     // update watch expressions
-    KTreeViewItem* item = m_watchVariables.itemAt(0);
-    for (; item != 0; item = item->getSibling()) {
-	m_watchEvalExpr.append(static_cast<VarTree*>(item));
+    VarTree* item = m_watchVariables.firstChild();
+    for (; item != 0; item = item->nextSibling()) {
+	m_watchEvalExpr.push_back(item->getText());
     }
 }
 
@@ -1293,7 +1294,7 @@ void KDebugger::handleLocals(const char* output)
     /*
      *  Get local variables.
      */
-    QList<VarTree> newVars;
+    QList<ExprValue> newVars;
     parseLocals(output, newVars);
 
     /*
@@ -1302,62 +1303,50 @@ void KDebugger::handleLocals(const char* output)
      */
     m_localVariables.clearPendingUpdates();
 
-    // reduce flicker
-    bool autoU = m_localVariables.autoUpdate();
-    m_localVariables.setAutoUpdate(false);
-    bool repaintNeeded = false;
-
     /*
      * Match old variables against new ones.
      */
     for (const char* n = oldVars.first(); n != 0; n = oldVars.next()) {
 	// lookup this variable in the list of new variables
-	VarTree* v = newVars.first();
-	while (v != 0 && strcmp(v->getText(), n) != 0) {
+	ExprValue* v = newVars.first();
+	while (v != 0 && v->m_name != n) {
 	    v = newVars.next();
 	}
 	if (v == 0) {
 	    // old variable not in the new variables
 	    TRACE(QString("old var deleted: ") + n);
-	    v = m_localVariables.topLevelExprByName(n);
+	    VarTree* v = m_localVariables.topLevelExprByName(n);
 	    if (v != 0) {
 		m_localVariables.removeExpr(v);
-		repaintNeeded = true;
 	    }
 	} else {
 	    // variable in both old and new lists: update
 	    TRACE(QString("update var: ") + n);
-	    m_localVariables.updateExpr(newVars.current());
+	    m_localVariables.updateExpr(newVars.current(), *m_typeTable);
 	    // remove the new variable from the list
 	    newVars.remove();
 	    delete v;
-	    repaintNeeded = true;
 	}
     }
     // insert all remaining new variables
-    for (VarTree* v = newVars.first(); v != 0; v = newVars.next()) {
-	TRACE("new var: " + v->getText());
-	m_localVariables.insertExpr(v);
-	repaintNeeded = true;
+    while (!newVars.isEmpty())
+    {
+	ExprValue* v = newVars.take(0);
+	TRACE("new var: " + v->m_name);
+	m_localVariables.insertExpr(v, *m_typeTable);
+	delete v;
     }
-
-    // repaint
-    m_localVariables.setAutoUpdate(autoU);
-    if (repaintNeeded && autoU && m_localVariables.isVisible())
-	m_localVariables.repaint();
 }
 
-void KDebugger::parseLocals(const char* output, QList<VarTree>& newVars)
+void KDebugger::parseLocals(const char* output, QList<ExprValue>& newVars)
 {
-    QList<VarTree> vars;
+    QList<ExprValue> vars;
     m_d->parseLocals(output, vars);
 
     QString origName;			/* used in renaming variables */
     while (vars.count() > 0)
     {
-	VarTree* variable = vars.take(0);
-	// get some types
-	variable->inferTypesOfChildren(*m_typeTable);
+	ExprValue* variable = vars.take(0);
 	/*
 	 * When gdb prints local variables, those from the innermost block
 	 * come first. We run through the list of already parsed variables
@@ -1368,13 +1357,13 @@ void KDebugger::parseLocals(const char* output, QList<VarTree>& newVars)
 	 * _visible_ changes the color!
 	 */
 	int block = 0;
-	origName = variable->getText();
-	for (VarTree* v = newVars.first(); v != 0; v = newVars.next()) {
-	    if (variable->getText() == v->getText()) {
+	origName = variable->m_name;
+	for (ExprValue* v = newVars.first(); v != 0; v = newVars.next()) {
+	    if (variable->m_name == v->m_name) {
 		// we found a duplicate, change name
 		block++;
 		QString newName = origName + " (" + QString().setNum(block) + ")";
-		variable->setText(newName);
+		variable->m_name = newName;
 	    }
 	}
 	newVars.append(variable);
@@ -1385,16 +1374,16 @@ bool KDebugger::handlePrint(CmdQueueItem* cmd, const char* output)
 {
     ASSERT(cmd->m_expr != 0);
 
-    VarTree* variable = parseExpr(output, true);
+    ExprValue* variable = m_d->parsePrintExpr(output, true);
     if (variable == 0)
 	return false;
 
     // set expression "name"
-    variable->setText(cmd->m_expr->getText());
+    variable->m_name = cmd->m_expr->getText();
 
     {
 	TRACE("update expr: " + cmd->m_expr->getText());
-	cmd->m_exprWnd->updateExpr(cmd->m_expr, variable);
+	cmd->m_exprWnd->updateExpr(cmd->m_expr, variable, *m_typeTable);
 	delete variable;
     }
 
@@ -1407,31 +1396,30 @@ bool KDebugger::handlePrintDeref(CmdQueueItem* cmd, const char* output)
 {
     ASSERT(cmd->m_expr != 0);
 
-    VarTree* variable = parseExpr(output, true);
+    ExprValue* variable = m_d->parsePrintExpr(output, true);
     if (variable == 0)
 	return false;
 
     // set expression "name"
-    variable->setText(cmd->m_expr->getText());
+    variable->m_name = cmd->m_expr->getText();
 
     {
 	/*
 	 * We must insert a dummy parent, because otherwise variable's value
 	 * would overwrite cmd->m_expr's value.
 	 */
-	VarTree* dummyParent = new VarTree(variable->getText(), VarTree::NKplain);
+	ExprValue* dummyParent = new ExprValue(variable->m_name, VarTree::NKplain);
 	dummyParent->m_varKind = VarTree::VKdummy;
 	// the name of the parsed variable is the address of the pointer
-	QString addr = "*" + cmd->m_expr->m_value;
-	variable->setText(addr);
+	QString addr = "*" + cmd->m_expr->value();
+	variable->m_name = addr;
 	variable->m_nameKind = VarTree::NKaddress;
 
-	dummyParent->appendChild(variable);
-	dummyParent->setDeleteChildren(true);
+	dummyParent->m_child = variable;
 	// expand the first level for convenience
-	variable->setExpanded(true);
+	variable->m_initiallyExpanded = true;
 	TRACE("update ptr: " + cmd->m_expr->getText());
-	cmd->m_exprWnd->updateExpr(cmd->m_expr, dummyParent);
+	cmd->m_exprWnd->updateExpr(cmd->m_expr, dummyParent, *m_typeTable);
 	delete dummyParent;
     }
 
@@ -1440,21 +1428,6 @@ bool KDebugger::handlePrintDeref(CmdQueueItem* cmd, const char* output)
     return true;
 }
 
-VarTree* KDebugger::parseExpr(const char* output, bool wantErrorValue)
-{
-    VarTree* variable;
-
-    // check for error conditions
-    bool goodValue = m_d->parsePrintExpr(output, wantErrorValue, variable);
-
-    if (variable != 0 && goodValue)
-    {
-	// get some types
-	variable->inferTypesOfChildren(*m_typeTable);
-    }
-    return variable;
-}
-
 // parse the output of bt
 void KDebugger::handleBacktrace(const char* output)
 {
@@ -1475,7 +1448,7 @@ void KDebugger::handleBacktrace(const char* output)
 	do {
 	    QString func;
 	    if (frm->var != 0)
-		func = frm->var->getText();
+		func = frm->var->m_name;
 	    else
 		func = frm->fileName + ":" + QString().setNum(frm->lineNo+1);
 	    m_btWindow.insertItem(func);
@@ -1517,21 +1490,22 @@ void KDebugger::evalExpressions()
     //   pointers in watch expressions
     //   types in local variables
     //   types in watch expressions
-    //   pointers in 'this'
-    //   types in 'this'
-    
-    VarTree* exprItem = m_watchEvalExpr.first();
+    //   struct members in local variables
+    //   struct members in watch expressions
+    VarTree* exprItem = 0;
+    if (!m_watchEvalExpr.empty())
+    {
+	QString expr = m_watchEvalExpr.front();
+	m_watchEvalExpr.pop_front();
+	exprItem = m_watchVariables.topLevelExprByName(expr);
+    }
     if (exprItem != 0) {
-	m_watchEvalExpr.remove();
-	QString expr = exprItem->computeExpr();
-	TRACE("watch expr: " + expr);
-	CmdQueueItem* cmd = m_d->queueCmd(DCprint, expr, DebuggerDriver::QMoverride);
+	CmdQueueItem* cmd = m_d->queueCmd(DCprint, exprItem->getText(), DebuggerDriver::QMoverride);
 	// remember which expr this was
 	cmd->m_expr = exprItem;
 	cmd->m_exprWnd = &m_watchVariables;
     } else {
 	ExprWnd* wnd;
-	VarTree* exprItem;
 #define POINTER(widget) \
 		wnd = &widget; \
 		exprItem = widget.nextUpdatePtr(); \
@@ -1638,7 +1612,7 @@ void KDebugger::handleFindType(CmdQueueItem* cmd, const char* output)
 	    }
 	}
 	if (info == 0) {
-	    TRACE("unknown type");
+            TRACE("unknown type "+type);
 	    cmd->m_expr->m_type = TypeInfo::unknownType();
 	} else {
 	    cmd->m_expr->m_type = info;
@@ -1657,16 +1631,18 @@ void KDebugger::handlePrintStruct(CmdQueueItem* cmd, const char* output)
     ASSERT(var != 0);
     ASSERT(var->m_varKind == VarTree::VKstruct);
 
-    VarTree* partExpr;
-    if (cmd->m_cmd != DCprintQStringStruct) {
-	partExpr = parseExpr(output, false);
-    } else {
+    ExprValue* partExpr;
+    if (cmd->m_cmd == DCprintQStringStruct) {
 	partExpr = m_d->parseQCharArray(output, false, m_typeTable->qCharIsShort());
+    } else if (cmd->m_cmd == DCprintWChar) {
+	partExpr = m_d->parseQCharArray(output, false, true);
+    } else {
+	partExpr = m_d->parsePrintExpr(output, false);
     }
     bool errorValue =
 	partExpr == 0 ||
 	/* we only allow simple values at the moment */
-	partExpr->childCount() != 0;
+	partExpr->m_child != 0;
 
     QString partValue;
     if (errorValue)
@@ -1726,9 +1702,23 @@ void KDebugger::handlePrintStruct(CmdQueueItem* cmd, const char* output)
 void KDebugger::evalInitialStructExpression(VarTree* var, ExprWnd* wnd, bool immediate)
 {
     var->m_exprIndex = 0;
-    var->m_exprIndexUseGuard = true;
-    var->m_partialValue = var->m_type->m_displayString[0];
-    evalStructExpression(var, wnd, immediate);
+    if (var->m_type != TypeInfo::wchartType())
+    {
+	var->m_exprIndexUseGuard = true;
+	var->m_partialValue = var->m_type->m_displayString[0];
+	evalStructExpression(var, wnd, immediate);
+    }
+    else
+    {
+	var->m_exprIndexUseGuard = false;
+	QString expr = var->computeExpr();
+	CmdQueueItem* cmd = m_d->queueCmd(DCprintWChar, expr,
+				immediate  ?  DebuggerDriver::QMoverrideMoreEqual
+				: DebuggerDriver::QMoverride);
+	// remember which expression this was
+	cmd->m_expr = var;
+	cmd->m_exprWnd = wnd;
+    }
 }
 
 /* queues a printStruct command; var must have been initialized correctly */
@@ -1747,7 +1737,7 @@ void KDebugger::evalStructExpression(VarTree* var, ExprWnd* wnd, bool immediate)
 	exprFmt = var->m_type->m_exprStrings[var->m_exprIndex];
     }
 
-    SIZED_QString(expr, exprFmt.length() + base.length() + 10);
+    QString expr;
     expr.sprintf(exprFmt, base.data());
 
     DbgCommand dbgCmd = DCprintStruct;
@@ -1802,22 +1792,13 @@ CmdQueueItem* KDebugger::loadCoreFile()
     return m_d->queueCmd(DCcorefile, m_corefile, DebuggerDriver::QMoverride);
 }
 
-void KDebugger::slotLocalsExpanding(KTreeViewItem* item, bool& allow)
-{
-    exprExpandingHelper(&m_localVariables, item, allow);
-}
-
-void KDebugger::slotWatchExpanding(KTreeViewItem* item, bool& allow)
-{
-    exprExpandingHelper(&m_watchVariables, item, allow);
-}
-
-void KDebugger::exprExpandingHelper(ExprWnd* wnd, KTreeViewItem* item, bool&)
+void KDebugger::slotExpanding(QListViewItem* item)
 {
     VarTree* exprItem = static_cast<VarTree*>(item);
     if (exprItem->m_varKind != VarTree::VKpointer) {
 	return;
     }
+    ExprWnd* wnd = static_cast<ExprWnd*>(item->listView());
     dereferencePointer(wnd, exprItem, true);
 }
 
@@ -1825,14 +1806,15 @@ void KDebugger::exprExpandingHelper(ExprWnd* wnd, KTreeViewItem* item, bool&)
 void KDebugger::addWatch(const QString& t)
 {
     QString expr = t.stripWhiteSpace();
-    if (expr.isEmpty())
+    // don't add a watched expression again
+    if (expr.isEmpty() || m_watchVariables.topLevelExprByName(expr) != 0)
 	return;
-    VarTree* exprItem = new VarTree(expr, VarTree::NKplain);
-    m_watchVariables.insertExpr(exprItem);
+    ExprValue e(expr, VarTree::NKplain);
+    m_watchVariables.insertExpr(&e, *m_typeTable);
 
     // if we are boring ourselves, send down the command
     if (m_programActive) {
-	m_watchEvalExpr.append(exprItem);
+	m_watchEvalExpr.push_back(expr);
 	if (m_d->isIdle()) {
 	    evalExpressions();
 	}
@@ -1847,17 +1829,14 @@ void KDebugger::slotDeleteWatch()
     if (!m_d->isIdle())
 	return;
 
-    int index = m_watchVariables.currentItem();
-    if (index < 0)
-	return;
-    
-    VarTree* item = static_cast<VarTree*>(m_watchVariables.itemAt(index));
-    if (!item->isToplevelExpr())
+    VarTree* item = m_watchVariables.currentItem();
+    if (item == 0 || !item->isToplevelExpr())
 	return;
 
     // remove the variable from the list to evaluate
-    if (m_watchEvalExpr.findRef(item) >= 0) {
-	m_watchEvalExpr.remove();
+    QStringList::iterator i = m_watchEvalExpr.find(item->getText());
+    if (i != m_watchEvalExpr.end()) {
+	m_watchEvalExpr.erase(i);
     }
     m_watchVariables.removeExpr(item);
     // item is invalid at this point!
@@ -2093,9 +2072,9 @@ void KDebugger::slotValuePopup(const QString& expr)
 
     // construct the tip
     QString tip = v->getText() + " = ";
-    if (!v->m_value.isEmpty())
+    if (!v->value().isEmpty())
     {
-	tip += v->m_value;
+	tip += v->value();
     }
     else
     {
@@ -2218,18 +2197,16 @@ void KDebugger::handleSetPC(const char* /*output*/)
     gotoFrame(0);
 }
 
-void KDebugger::slotValueEdited(int row, const QString& text)
+void KDebugger::slotValueEdited(VarTree* expr, const QString& text)
 {
     if (text.simplifyWhiteSpace().isEmpty())
 	return;			       /* no text entered: ignore request */
 
-    ASSERT(sender()->inherits("ExprWnd"));
-    ExprWnd* wnd = const_cast<ExprWnd*>(static_cast<const ExprWnd*>(sender()));
-    TRACE(QString().sprintf("Changing %s at row %d to ",
-			    wnd->name(), row) + text);
+    ExprWnd* wnd = static_cast<ExprWnd*>(expr->listView());
+    TRACE(QString().sprintf("Changing %s to ",
+			    wnd->name()) + text);
 
     // determine the lvalue to edit
-    VarTree* expr = static_cast<VarTree*>(wnd->itemAt(row));
     QString lvalue = expr->computeExpr();
     CmdQueueItem* cmd = m_d->executeCmd(DCsetvariable, lvalue, text);
     cmd->m_expr = expr;
