@@ -11,65 +11,69 @@
 #include <qscrollbar.h>
 #include <kapp.h>
 #include <kiconloader.h>		/* icons */
+#include <klocale.h>			/* i18n */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 #include "mydebug.h"
 
-VarTree::VarTree(const QString& name, NameKind aKind) :
-	KTreeViewItem(name),
-	m_varKind(VKsimple),
-	m_nameKind(aKind),
-	m_valueChanged(false),
+VarTree::VarTree(VarTree* parent, QListViewItem* after, ExprValue* v) :
+	QListViewItem(parent, after),
+	m_varKind(v->m_varKind),
+	m_nameKind(v->m_nameKind),
 	m_type(0),
 	m_exprIndex(0),
-	m_exprIndexUseGuard(false)
+	m_exprIndexUseGuard(false),
+	m_baseValue(v->m_value),
+	m_baseChanged(false),
+	m_structChanged(false)
 {
+    QListViewItem::setText(0, v->m_name);
+    updateValueText();
+    setExpandable(m_varKind == VarTree::VKpointer);
+    setOpen(v->m_initiallyExpanded);
+}
+
+VarTree::VarTree(ExprWnd* parent, QListViewItem* after, const QString& name) :
+	QListViewItem(parent, after),
+	m_varKind(VKsimple),
+	m_nameKind(VarTree::NKplain),
+	m_type(0),
+	m_exprIndex(0),
+	m_exprIndexUseGuard(false),
+	m_baseChanged(false),
+	m_structChanged(false)
+{
+    QListViewItem::setText(0, name);
 }
 
 VarTree::~VarTree()
 {
 }
 
-void VarTree::paintValue(QPainter* p)
+void VarTree::paintCell(QPainter* p, const QColorGroup& cg, int column, int width, int align)
 {
-    p->save();
-    int cellHeight = height(p->fontMetrics());
-    int textX = 2;
-    int textY = (cellHeight - p->fontMetrics().height()) / 2 +
-	p->fontMetrics().ascent();
-
-    if (m_valueChanged) {
-	p->setPen(red);
+    if (column == 1 && (m_baseChanged || m_structChanged)) {
+	QColorGroup cgChg = cg;
+	cgChg.setColor(QColorGroup::Text, Qt::red);
+	QListViewItem::paintCell(p, cgChg, column, width, align);
+    } else {
+	QListViewItem::paintCell(p, cg, column, width, align);
     }
-//    p->setBackgroundColor(cg.base());
-    p->drawText(textX, textY, m_value, m_value.length());
-    p->restore();
-}
-
-int VarTree::valueWidth()
-{
-    assert(owner != 0);
-    return owner->fontMetrics().width(m_value) + 4;
 }
 
 QString VarTree::computeExpr() const
 {
-    assert(getParent() != 0);
-    // just to be sure
-    if (getParent() == 0)
-	return QString();
-
     // top-level items are special
-    if (getParent()->getParent() == 0)
+    if (isToplevelExpr())
 	return getText();
 
     // get parent expr
-    VarTree* par = static_cast<VarTree*>(getParent());
+    VarTree* par = static_cast<VarTree*>(parent());
     QString parentExpr = par->computeExpr();
 
-    /* don't add this item's name if this is a base class sub-item */
-    if (m_nameKind == NKtype) {
+    // skip this item's name if it is a base class or anonymous struct or union
+    if (m_nameKind == NKtype || m_nameKind == NKanonymous) {
 	return parentExpr;
     }
     /* augment by this item's text */
@@ -114,14 +118,14 @@ QString VarTree::computeExpr() const
 
 bool VarTree::isToplevelExpr() const
 {
-    return getParent() != 0 && getParent()->getParent() == 0;
+    return parent() == 0;
 }
 
 bool VarTree::isAncestorEq(const VarTree* child) const
 {
-    const KTreeViewItem* c = child;
+    const QListViewItem* c = child;
     while (c != 0 && c != this) {
-	c = c->getParent();
+	c = c->parent();
     }
     return c != 0;
 }
@@ -129,11 +133,10 @@ bool VarTree::isAncestorEq(const VarTree* child) const
 bool VarTree::updateValue(const QString& newValue)
 {
     // check whether the value changed
-    bool prevValueChanged = m_valueChanged;
-    m_valueChanged = false;
-    if (m_value != newValue) {
-	m_value = newValue;
-	m_valueChanged = true;
+    bool prevValueChanged = m_baseChanged;
+    if ((m_baseChanged = m_baseValue != newValue)) {
+	m_baseValue = newValue;
+	updateValueText();
     }
     /*
      * We must repaint the cell if the value changed. If it did not change,
@@ -141,7 +144,35 @@ bool VarTree::updateValue(const QString& newValue)
      * because the color of the display must be changed (from red to
      * black).
      */
-    return m_valueChanged || prevValueChanged;
+    return m_baseChanged || prevValueChanged;
+}
+
+bool VarTree::updateStructValue(const QString& newValue)
+{
+    // check whether the value changed
+    bool prevValueChanged = m_structChanged;
+    if ((m_structChanged = m_structValue != newValue)) {
+	m_structValue = newValue;
+	updateValueText();
+    }
+    /*
+    * We must repaint the cell if the value changed. If it did not change,
+    * we still must repaint the cell if the value changed previously,
+    * because the color of the display must be changed (from red to
+    * black).
+    */
+    return m_structChanged || prevValueChanged;
+}
+
+void VarTree::updateValueText()
+{
+    if (m_baseValue.isEmpty()) {
+	QListViewItem::setText(1, m_structValue);
+    } else if (m_structValue.isEmpty()) {
+	QListViewItem::setText(1, m_baseValue);
+    } else {
+	QListViewItem::setText(1, m_baseValue + " " + m_structValue);
+    }
 }
 
 void VarTree::inferTypesOfChildren(ProgramTypeTable& typeTable)
@@ -153,45 +184,29 @@ void VarTree::inferTypesOfChildren(ProgramTypeTable& typeTable)
      */
 
     // first recurse children
-    VarTree* child = static_cast<VarTree*>(getChild());
+    VarTree* child = firstChild();
     while (child != 0) {
 	child->inferTypesOfChildren(typeTable);
-	child = static_cast<VarTree*>(child->getSibling());
+	child = child->nextSibling();
     }
 
     // if this is a pointer, get the type from the value (less the pointer)
     if (m_varKind == VKpointer) {
-#ifndef I_know_a_way_to_do_this_cleanly
-	return;
-#else
-	const char* p = m_value.data();
-	const char* start = p;
-	// the type of the pointer shows up in the value (sometimes)
-	if (p == 0 || *p != '(')
-	    return;
-	skipNested(p, '(', ')');
-	/*
-	 * We only recognize pointers to data "(int *)" but not pointers
-	 * to functions "(void (*)())".
-	 */
-	if (p-start < 3 &&		/* at least 3 chars necessary: (*) */
-	    p[-2] != '*')		/* skip back before the closing paren */
+	if (isWcharT())
 	{
-	    return;
+	    /*
+	     * wchart_t pointers must be treated as struct, because the array
+	     * of characters is printed similar to how QStrings are decoded.
+	     */
+	    m_varKind = VKstruct;
+	    setExpandable(false);
 	}
-	const QString& typeName =
-	    FROM_LATIN1(start+1, p-start-3) // minus 3 chars
-		.stripWhiteSpace();
-	m_type = typeTable.lookup(typeName);
-	if (m_type == 0) {
-	    m_type = TypeInfo::unknownType();
-	}
-#endif
+	// don't know how to do this cleanly
     } else if (m_varKind == VKstruct) {
 	// check if this is a base class part
 	if (m_nameKind == NKtype) {
 	    const QString& typeName =
-		text.mid(1, text.length()-2); // strip < and >
+		getText().mid(1, getText().length()-2);	// strip < and >
 	    m_type = typeTable.lookup(typeName);
 
 	    /* if we don't have a type yet, get it from the base class */
@@ -217,13 +232,20 @@ void VarTree::inferTypesOfChildren(ProgramTypeTable& typeTable)
     }
 }
 
+// the value contains the pointer type in parenthesis
+bool VarTree::isWcharT() const
+{
+    return value().startsWith("(const wchar_t *)") ||
+	    value().startsWith("(wchar_t *)");
+}
+
 /*
  * Get the type of the first base class whose type we know.
  */
 TypeInfo* VarTree::inferTypeFromBaseClass()
 {
     if (m_varKind == VKstruct) {
-	VarTree* child = static_cast<VarTree*>(getChild());
+	VarTree* child = firstChild();
 	while (child != 0 &&
 	       // only check base class parts (i.e. type names)
 	       child->m_nameKind == NKtype)
@@ -234,22 +256,66 @@ TypeInfo* VarTree::inferTypeFromBaseClass()
 		// got a type!
 		return child->m_type;
 	    }
-	    child = static_cast<VarTree*>(child->getSibling());
+	    child = child->nextSibling();
 	}
     }
     return 0;
 }
 
-
-ExprWnd::ExprWnd(QWidget* parent, const char* name) :
-	KTreeView(parent, name),
-	maxValueWidth(0),
-	m_edit(this)
+ExprValue::ExprValue(const QString& name, VarTree::NameKind aKind) :
+	m_name(name),
+	m_varKind(VarTree::VKsimple),
+	m_nameKind(aKind),
+	m_child(0),
+	m_next(0),
+	m_initiallyExpanded(false)
 {
-    setNumCols(2);
-    
-    connect(this, SIGNAL(expanded(int)), SLOT(slotExpandOrCollapse(int)));
-    connect(this, SIGNAL(collapsed(int)), SLOT(slotExpandOrCollapse(int)));
+}
+
+ExprValue::~ExprValue()
+{
+    delete m_child;
+    delete m_next;
+}
+
+void ExprValue::appendChild(ExprValue* newChild)
+{
+    if (m_child == 0) {
+	m_child = newChild;
+    } else {
+	// walk chain of children to find the last one
+	ExprValue* last = m_child;
+	while (last->m_next != 0)
+	    last = last->m_next;
+	last->m_next = newChild;
+    }
+    newChild->m_next = 0;	// just to be sure
+}
+
+int ExprValue::childCount() const
+{
+    int i = 0;
+    ExprValue* c = m_child;
+    while (c) {
+	++i;
+	c = c->m_next;
+    }
+    return i;
+}
+
+
+
+ExprWnd::ExprWnd(QWidget* parent, const QString& colHeader, const char* name) :
+	QListView(parent, name),
+	m_edit(0)
+{
+    addColumn(colHeader);
+    addColumn(i18n("Value"));
+    setSorting(-1);		// do not sort items
+    setColumnWidthMode(0, Manual);
+    setColumnWidthMode(1, Maximum);
+    setRootIsDecorated(true);
+    setAllColumnsShowFocus(true);
 
     m_pixPointer = UserIcon("pointer.xpm");
     if (m_pixPointer.isNull())
@@ -263,56 +329,52 @@ ExprWnd::~ExprWnd()
 void ExprWnd::exprList(QStrList& exprs)
 {
     // ASSERT(exprs does deep-copies)
-    KTreeViewItem* item;
-    for (item = itemAt(0); item != 0; item = item->getSibling()) {
+    VarTree* item;
+    for (item = firstChild(); item != 0; item = item->nextSibling()) {
 	exprs.append(item->getText());
     }
 }
 
-void ExprWnd::insertExpr(VarTree* expr)
+VarTree* ExprWnd::insertExpr(ExprValue* expr, ProgramTypeTable& typeTable)
 {
-    // append the expression
-    insertItem(expr);
+    // append a new dummy expression
+    VarTree* last = 0;	// last top-level item
+    for (VarTree* i = firstChild(); i != 0; i = i->nextSibling()) {
+	last = i;
+    }
+    VarTree* display = new VarTree(this, last, expr->m_name);
 
-    collectUnknownTypes(expr);
-
-    updateValuesWidth();
+    // replace it right away
+    updateExpr(display, expr, typeTable);
+    return display;
 }
 
-void ExprWnd::updateExpr(VarTree* expr)
+void ExprWnd::updateExpr(ExprValue* expr, ProgramTypeTable& typeTable)
 {
     // search the root variable
-    KPath path;
-    path.push(expr->getText());
-    KTreeViewItem* item = itemAt(path);
+    VarTree* item = firstChild();
+    while (item != 0 && item->getText() != expr->m_name)
+	item = item->nextSibling();
     if (item == 0) {
 	return;
     }
     // now update it
-    if (updateExprRec(static_cast<VarTree*>(item), expr)) {
-	updateVisibleItems();
-	updateValuesWidth();
-	repaint();
-    }
-    collectUnknownTypes(static_cast<VarTree*>(item));
+    updateExprRec(item, expr, typeTable);
+    collectUnknownTypes(item);
 }
 
-void ExprWnd::updateExpr(VarTree* display, VarTree* newValues)
+void ExprWnd::updateExpr(VarTree* display, ExprValue* newValues, ProgramTypeTable& typeTable)
 {
-    if (updateExprRec(display, newValues) && display->isVisible()) {
-	updateVisibleItems();
-	updateValuesWidth();
-	repaint();
-    }
+    updateExprRec(display, newValues, typeTable);
     collectUnknownTypes(display);
 }
 
 /*
  * returns true if there's a visible change
  */
-bool ExprWnd::updateExprRec(VarTree* display, VarTree* newValues)
+void ExprWnd::updateExprRec(VarTree* display, ExprValue* newValues, ProgramTypeTable& typeTable)
 {
-    bool isExpanded = display->isExpanded();
+    bool isExpanded = display->isOpen();
 
     /*
      * If we are updating a pointer without children by a dummy, we don't
@@ -324,7 +386,7 @@ bool ExprWnd::updateExprRec(VarTree* display, VarTree* newValues)
 	newValues->m_varKind == VarTree::VKdummy)
     {
 	replaceChildren(display, newValues);
-	return isExpanded;		/* no visible change if not expanded */
+	return;
     }
 
     /*
@@ -342,10 +404,10 @@ bool ExprWnd::updateExprRec(VarTree* display, VarTree* newValues)
 	  * sub-tree for requiring an update.
 	  */
 	 (display->m_varKind != VarTree::VKpointer ||
-	  newValues->childCount() != 0)))
+	  newValues->m_child != 0)))
     {
 	if (isExpanded) {
-	    collapseSubTree(display, false);
+	    display->setOpen(false);
 	}
 
 	// since children changed, it is likely that the type has also changed
@@ -358,11 +420,14 @@ bool ExprWnd::updateExprRec(VarTree* display, VarTree* newValues)
 	// update the m_varKind
 	if (newValues->m_varKind != VarTree::VKdummy) {
 	    display->m_varKind = newValues->m_varKind;
-	    display->setDelayedExpanding(newValues->m_varKind == VarTree::VKpointer);
+	    display->setExpandable(newValues->m_varKind == VarTree::VKpointer);
 	}
 
+	// get some types (after the new m_varKind has been set!)
+	display->inferTypesOfChildren(typeTable);
+
 	// (note that the new value might not have a sub-tree at all)
-	return display->m_valueChanged || isExpanded;	/* no visible change if not expanded */
+	return;
     }
 
     // display the new value
@@ -382,42 +447,31 @@ bool ExprWnd::updateExprRec(VarTree* display, VarTree* newValues)
 	 * If the visible sub-tree has children, but newValues doesn't, we
 	 * can stop here.
 	 */
-	if (newValues->childCount() == 0) {
-	    return display->m_valueChanged;
+	if (newValues->m_child == 0) {
+	    return;
 	}
     }
 
     ASSERT(display->childCount() == newValues->childCount());
 
     // go for children
-    bool childChanged = false;
-
-    VarTree* vDisplay = static_cast<VarTree*>(display->getChild());
-    VarTree* vNew = static_cast<VarTree*>(newValues->getChild());
+    VarTree* vDisplay = display->firstChild();
+    ExprValue* vNew = newValues->m_child;
     while (vDisplay != 0) {
 	// check whether the names are the same
-	if (strcmp(vDisplay->getText(), vNew->getText()) != 0) {
+	if (vDisplay->getText() != vNew->m_name) {
 	    // set new name
-	    vDisplay->setText(vNew->getText());
-	    int i = itemRow(vDisplay);
-	    if (i >= 0) {
-		updateCell(i, 0, true);
-		childChanged = true;
-	    }
+	    vDisplay->setText(vNew->m_name);
 	}
 	// recurse
-	if (updateExprRec(vDisplay, vNew)) {
-	    childChanged = true;
-	}
-	vDisplay = static_cast<VarTree*>(vDisplay->getSibling());
-	vNew = static_cast<VarTree*>(vNew->getSibling());
-    }
+	updateExprRec(vDisplay, vNew, typeTable);
 
-    // update of children propagates only if this node is expanded
-    return display->m_valueChanged || (display->isExpanded() && childChanged);
+	vDisplay = vDisplay->nextSibling();
+	vNew = vNew->m_next;
+    }
 }
 
-void ExprWnd::updateSingleExpr(VarTree* display, VarTree* newValue)
+void ExprWnd::updateSingleExpr(VarTree* display, ExprValue* newValue)
 {
     /*
      * If newValues is a VKdummy, we are only interested in its children.
@@ -428,26 +482,28 @@ void ExprWnd::updateSingleExpr(VarTree* display, VarTree* newValue)
     }
 
     /*
-     * If this node is a struct and we know its type then don't update its
-     * value now. This is a node for which we know how to find a nested
-     * value. So register the node for an update.
+     * If this node is a struct and we know its type then we know how to
+     * find a nested value. So register the node for an update.
+     *
+     * wchar_t types are also treated specially here: We consider them
+     * as struct (has been set in inferTypesOfChildren()).
      */
     if (display->m_varKind == VarTree::VKstruct &&
 	display->m_type != 0 &&
 	display->m_type != TypeInfo::unknownType())
     {
 	ASSERT(newValue->m_varKind == VarTree::VKstruct);
-	display->m_partialValue = display->m_type->m_displayString[0];
+	if (display->m_type == TypeInfo::wchartType())
+	{
+	    display->m_partialValue = "L";
+	}
+	else
+	    display->m_partialValue = display->m_type->m_displayString[0];
 	m_updateStruct.append(display);
     }
-    else
-    {
-	if (display->updateValue(newValue->m_value)) {
-	    int i = itemRow(display);
-	    if (i >= 0) {
-		updateCell(i, 1, true);
-	    }
-	}
+
+    if (display->updateValue(newValue->m_value)) {
+	triggerUpdate();
     }
 }
 
@@ -455,38 +511,28 @@ void ExprWnd::updateStructValue(VarTree* display)
 {
     ASSERT(display->m_varKind == VarTree::VKstruct);
 
-    if (display->updateValue(display->m_partialValue)) {
-	int i = itemRow(display);
-	if (i >= 0) {
-	    updateValuesWidth();
-	    updateCell(i, 1, true);
-	}
+    if (display->updateStructValue(display->m_partialValue)) {
+	triggerUpdate();
     }
     // reset the value
     display->m_partialValue = "";
     display->m_exprIndex = -1;
 }
 
-void ExprWnd::replaceChildren(VarTree* display, VarTree* newValues)
+void ExprWnd::replaceChildren(VarTree* display, ExprValue* newValues)
 {
     ASSERT(display->childCount() == 0 || display->m_varKind != VarTree::VKsimple);
 
     // delete all children of display
-    while (VarTree* c = static_cast<VarTree*>(display->getChild())) {
+    while (VarTree* c = display->firstChild()) {
 	unhookSubtree(c);
-	display->removeChild(c);
 	delete c;
     }
     // insert copies of the newValues
-    for (KTreeViewItem* c = newValues->getChild(); c != 0; c = c->getSibling()) {
-	VarTree* v = static_cast<VarTree*>(c);
-	VarTree* vNew = new VarTree(v->getText(), v->m_nameKind);
-	vNew->m_varKind = v->m_varKind;
-	vNew->m_value = v->m_value;
-	vNew->m_type = v->m_type;
-	vNew->setDelayedExpanding(vNew->m_varKind == VarTree::VKpointer);
-	vNew->setExpanded(v->isExpanded());
-	display->appendChild(vNew);
+    VarTree* vNew = 0;
+    for (ExprValue* v = newValues->m_child; v != 0; v = v->m_next)
+    {
+	vNew = new VarTree(display, vNew, v);
 	// recurse
 	replaceChildren(vNew, v);
     }
@@ -494,54 +540,55 @@ void ExprWnd::replaceChildren(VarTree* display, VarTree* newValues)
 
 void ExprWnd::collectUnknownTypes(VarTree* var)
 {
-    /*
-     * forEveryItem does not scan the root item itself. So we must do it
-     * ourselves.
-     */
+    QListViewItemIterator i(var);
+    for (; i.current(); ++i)
+    {
+	checkUnknownType(static_cast<VarTree*>(i.current()));
+    }
+}
+
+void ExprWnd::checkUnknownType(VarTree* var)
+{
     ASSERT(var->m_varKind != VarTree::VKpointer || var->m_nameKind != VarTree::NKtype);
     if (var->m_type == 0 &&
 	var->m_varKind == VarTree::VKstruct &&
-	var->m_nameKind != VarTree::NKtype)
+	var->m_nameKind != VarTree::NKtype &&
+	var->m_nameKind != VarTree::NKanonymous)
     {
-	/* this struct node doesn't have a type yet: register it */
-	m_updateType.append(var);
+	if (!var->isWcharT())
+	{
+	    /* this struct node doesn't have a type yet: register it */
+	    m_updateType.append(var);
+	}
+	else
+	{
+	    var->m_type = TypeInfo::wchartType();
+	    var->m_partialValue = "L";
+	    m_updateStruct.append(var);
+	}
     }
-
     // add pointer pixmap to pointers
     if (var->m_varKind == VarTree::VKpointer) {
 	var->setPixmap(m_pixPointer);
     }
-
-    forEveryItem(collectUnknownTypes, this, var);
 }
 
-bool ExprWnd::collectUnknownTypes(KTreeViewItem* item, void* user)
+QString ExprWnd::formatWCharPointer(QString value)
 {
-    VarTree* var = static_cast<VarTree*>(item);
-    ExprWnd* tree = static_cast<ExprWnd*>(user);
-    ASSERT(var->m_varKind != VarTree::VKpointer || var->m_nameKind != VarTree::NKtype);
-    if (var->m_type == 0 &&
-	var->m_varKind == VarTree::VKstruct &&
-	var->m_nameKind != VarTree::NKtype)
-    {
-	/* this struct node doesn't have a type yet: register it */
-	tree->m_updateType.append(var);
-    }
-    // add pointer pixmap to pointers
-    if (var->m_varKind == VarTree::VKpointer) {
-	var->setPixmap(tree->m_pixPointer);
-    }
-    return false;
+    int pos = value.find(") ");
+    if (pos > 0)
+	value = value.mid(pos+2);
+    return value + " L";
 }
 
 
 VarTree* ExprWnd::topLevelExprByName(const char* name)
 {
-    KPath path;
-    path.push(name);
-    KTreeViewItem* item = itemAt(path);
+    VarTree* item = firstChild();
+    while (item != 0 && item->getText() != name)
+	item = item->nextSibling();
 
-    return static_cast<VarTree*>(item);
+    return item;
 }
 
 VarTree* ExprWnd::ptrMemberByName(VarTree* v, const QString& name)
@@ -551,29 +598,32 @@ VarTree* ExprWnd::ptrMemberByName(VarTree* v, const QString& name)
 	return 0;
 
     // the only child of v is the pointer value that represents the struct
-    KTreeViewItem* item = v->getChild();
-    return memberByName(static_cast<VarTree*>(item), name);
+    VarTree* item = v->firstChild();
+    return memberByName(item, name);
 }
 
 VarTree* ExprWnd::memberByName(VarTree* v, const QString& name)
 {
     // search immediate children for name
-    KTreeViewItem* item = v->getChild();
+    VarTree* item = v->firstChild();
     while (item != 0 && item->getText() != name)
-	item = item->getSibling();
+	item = item->nextSibling();
 
     if (item != 0)
-	return static_cast<VarTree*>(item);
+	return item;
 
-    // try in base classes
-    item = v->getChild();
-    while (item != 0 &&
-	   static_cast<VarTree*>(item)->m_nameKind == VarTree::NKtype)
+    // try in base classes and members that are anonymous structs or unions
+    item = v->firstChild();
+    while (item != 0)
     {
-	v = memberByName(static_cast<VarTree*>(item), name);
-	if (v != 0)
-	    return v;
-	item = item->getSibling();
+	if (item->m_nameKind == VarTree::NKtype ||
+	    item->m_nameKind == VarTree::NKanonymous)
+	{
+	    v = memberByName(item, name);
+	    if (v != 0)
+		return v;
+	}
+	item = item->nextSibling();
     }
     return 0;
 }
@@ -582,10 +632,7 @@ void ExprWnd::removeExpr(VarTree* item)
 {
     unhookSubtree(item);
 
-    takeItem(item);
     delete item;
-
-    updateValuesWidth();
 }
 
 void ExprWnd::unhookSubtree(VarTree* subTree)
@@ -628,14 +675,6 @@ void ExprWnd::unhookSubtree(QList<VarTree>& list, VarTree* subTree)
     }
 }
 
-QString ExprWnd::exprStringAt(int index)
-{
-    KTreeViewItem* item = itemAt(index);
-    if (item == 0) return QString();	/* paranoia */
-    VarTree* expr = static_cast<VarTree*>(item);
-    return expr->computeExpr();
-}
-
 void ExprWnd::clearPendingUpdates()
 {
     m_updatePtrs.clear();
@@ -670,60 +709,18 @@ VarTree* ExprWnd::nextUpdateStruct()
     return ptr;
 }
 
-void ExprWnd::paintCell(QPainter* painter, int row, int col)
-{
-    if (col == 0) {
-	KTreeView::paintCell(painter, row, col);
-    } else {
-	VarTree* item = static_cast<VarTree*>(itemAt(row));
-	if (item != 0) {
-	    item->paintValue(painter);
-	}
-    }
-}
 
-int ExprWnd::cellWidth(int col) const
+void ExprWnd::editValue(VarTree* item, const QString& text)
 {
-    if (col == 0) {
-	return KTreeView::cellWidth(col);
-    } else {
-	return maxValueWidth;
-    }
-}
+    if (m_edit == 0)
+	m_edit = new ValueEdit(this);
 
-void ExprWnd::updateValuesWidth()
-{
-    int maxW = 0;
-    forEveryVisibleItem(static_cast<KForEveryFunc>(&getMaxValueWidth), &maxW);
-    maxValueWidth = maxW;
-    updateTableSize();
-}
-
-// called by updateValuesWidth() for each item in the visible list
-bool ExprWnd::getMaxValueWidth(KTreeViewItem* item, void* user)
-{
-    int *maxW = (int *)user;
-    VarTree* v = static_cast<VarTree*>(item);
-    int w = v->valueWidth();
-    if(w > *maxW)
-	*maxW = w;
-    return false;
-}
-
-void ExprWnd::slotExpandOrCollapse(int)
-{
-    updateValuesWidth();
-}
-
-void ExprWnd::editValue(int row, const QString& text)
-{
-    int x;
-    colXPos(1, &x);
-    int y;
-    rowYPos(row, &y);
-    int w = cellWidth(1);
-    int h = cellHeight(row);
-    QScrollBar* sbV = static_cast<QScrollBar*>(child("table_sbV"));
+    QRect r = itemRect(item);
+    int x = r.x()+columnWidth(0);
+    int y = r.y();
+    int w = columnWidth(1);
+    int h = r.height();
+    QListView* lv = item->listView();
 
     /*
      * Make the edit widget at least 5 characters wide (but not wider than
@@ -731,20 +728,18 @@ void ExprWnd::editValue(int row, const QString& text)
      * the text, scroll this widget so that half of it shows the text (or
      * less than half of it if the text is shorter).
      */
-    QFontMetrics metr = m_edit.font();
+    QFontMetrics metr = m_edit->font();
     int wMin = metr.width("88888");
     if (w < wMin)
 	w = wMin;
-    int wThis = width();
-    if (sbV->isVisible())	// subtract width of scrollbar
-	wThis -= sbV->width();
+    int wThis = lv->visibleWidth();
     if (x >= wThis/2 &&		// less than half the width displays text
 	x+w > wThis)		// not all text is visible
     {
 	// scroll so that more text is visible
 	int wScroll = QMIN(x-wThis/2, x+w-wThis);
-	sbHor(xOffset()+wScroll);
-	colXPos(1, &x);
+	lv->scrollBy(wScroll, 0);
+	x -= wScroll;
     }
     else if (x < 0)
     {
@@ -754,33 +749,34 @@ void ExprWnd::editValue(int row, const QString& text)
 
     // make the edit box as wide as the visible column
     QRect rect(x,y, wThis-x,h);
-    m_edit.setText(text);
-    m_edit.selectAll();
+    m_edit->setText(text);
+    m_edit->selectAll();
 
-    m_edit.setGeometry(rect);
-    m_edit.m_finished = false;
-    m_edit.m_row = row;
-    m_edit.show();
-    m_edit.setFocus();
+    m_edit->setGeometry(rect);
+    m_edit->m_finished = false;
+    m_edit->m_item = item;
+    m_edit->show();
+    m_edit->setFocus();
 }
 
 bool ExprWnd::isEditing() const
 {
-    return m_edit.isVisible();
+    return m_edit != 0 && m_edit->isVisible();
 }
 
 
 ValueEdit::ValueEdit(ExprWnd* parent) :
-	QLineEdit(parent, "valueedit")
+	QLineEdit(parent->viewport(), "valueedit")
 {
     setFrame(false);
     hide();
     lower();	// lower the window below scrollbars
-    connect(parent, SIGNAL(selected(int)), SLOT(slotSelectionChanged()));
-    connect(parent, SIGNAL(collapsed(int)), SLOT(slotSelectionChanged()));
-    connect(parent, SIGNAL(expanded(int)), SLOT(slotSelectionChanged()));
-    connect(this, SIGNAL(done(int, const QString&)),
-	    parent, SIGNAL(editValueCommitted(int, const QString&)));
+    connect(parent, SIGNAL(selectionChanged()), SLOT(slotSelectionChanged()));
+    connect(parent, SIGNAL(currentChanged(QListViewItem*)), SLOT(slotSelectionChanged()));
+    connect(parent, SIGNAL(expanded(QListViewItem*)), SLOT(slotSelectionChanged()));
+    connect(parent, SIGNAL(collapsed(QListViewItem*)), SLOT(slotSelectionChanged()));
+    connect(this, SIGNAL(done(VarTree*, const QString&)),
+	    parent, SIGNAL(editValueCommitted(VarTree*, const QString&)));
 }
 
 ValueEdit::~ValueEdit()
@@ -795,7 +791,7 @@ void ValueEdit::terminate(bool commit)
 	m_finished = true;
 	hide();	// will call focusOutEvent, that's why we need m_finished
 	if (commit) {
-	    emit done(m_row, text());
+	    emit done(m_item, text());
 	}
     }
 }
@@ -822,9 +818,17 @@ void ValueEdit::focusOutEvent(QFocusEvent* ev)
 {
     TRACE("ValueEdit::focusOutEvent");
     QFocusEvent* focusEv = static_cast<QFocusEvent*>(ev);
+    if (focusEv->reason() == QFocusEvent::ActiveWindow)
+    {
+	// Switching to a different window should terminate the edit,
+	// because if the window with this variable display is floating
+	// then that different window could be the main window, where
+	// the user had clicked one of the Execute buttons. This in turn
+	// may pull the item away that we are editing here.
+	terminate(false);
+    }
     // Don't let a RMB close the editor
-    if (focusEv->reason() != QFocusEvent::Popup &&
-	focusEv->reason() != QFocusEvent::ActiveWindow)
+    else if (focusEv->reason() != QFocusEvent::Popup)
     {
 	terminate(true);
     }
