@@ -20,12 +20,12 @@
 
 static void skipString(const char*& p);
 static void skipNested(const char*& s, char opening, char closing);
-static VarTree* parseVar(const char*& s);
+static ExprValue* parseVar(const char*& s);
 static bool parseName(const char*& s, QString& name, VarTree::NameKind& kind);
-static bool parseValue(const char*& s, VarTree* variable);
-static bool parseNested(const char*& s, VarTree* variable);
-static bool parseVarSeq(const char*& s, VarTree* variable);
-static bool parseValueSeq(const char*& s, VarTree* variable);
+static bool parseValue(const char*& s, ExprValue* variable);
+static bool parseNested(const char*& s, ExprValue* variable);
+static bool parseVarSeq(const char*& s, ExprValue* variable);
+static bool parseValueSeq(const char*& s, ExprValue* variable);
 
 #define PROMPT "(kdbg)"
 #define PROMPT_LEN 6
@@ -115,6 +115,7 @@ static GdbCmdInfo cmds[] = {
     { DCcondition, "condition %d %s\n", GdbCmdInfo::argNumString},
     { DCsetpc, "set variable $pc=%s\n", GdbCmdInfo::argString },
     { DCignore, "ignore %d %d\n", GdbCmdInfo::argNum2},
+    { DCprintWChar, "print ($s=%s)?*$s@wcslen($s):0x0\n", GdbCmdInfo::argString },
     { DCsetvariable, "set variable %s=%s\n", GdbCmdInfo::argString2 },
 };
 
@@ -301,7 +302,11 @@ void GdbDriver::commandFinished(CmdQueueItem* cmd)
 	    }
 	    // use a feasible core-file command
 	    if (m_gdbMajor > 4 || (m_gdbMajor == 4 && m_gdbMinor >= 16)) {
+#ifdef __FreeBSD__
+		cmds[DCcorefile].fmt = "target FreeBSD-core %s\n";
+#else
 		cmds[DCcorefile].fmt = "target core %s\n";
+#endif
 	    } else {
 		cmds[DCcorefile].fmt = "core-file %s\n";
 	    }
@@ -405,7 +410,7 @@ QString GdbDriver::makeCmdString(DbgCommand cmd, QString strArg)
 	strArg = m_redirect + " " + strArg;
     }
 
-    SIZED_QString(cmdString, MAX_FMTLEN+strArg.length());
+    QString cmdString;
     cmdString.sprintf(cmds[cmd].fmt, strArg.latin1());
     return cmdString;
 }
@@ -415,8 +420,7 @@ QString GdbDriver::makeCmdString(DbgCommand cmd, int intArg)
     assert(cmd >= 0 && cmd < NUM_CMDS);
     assert(cmds[cmd].argsNeeded == GdbCmdInfo::argNum);
 
-    SIZED_QString(cmdString, MAX_FMTLEN+30);
-
+    QString cmdString;
     cmdString.sprintf(cmds[cmd].fmt, intArg);
     return cmdString;
 }
@@ -431,7 +435,7 @@ QString GdbDriver::makeCmdString(DbgCommand cmd, QString strArg, int intArg)
 
     normalizeStringArg(strArg);
 
-    SIZED_QString(cmdString, MAX_FMTLEN+30+strArg.length());
+    QString cmdString;
 
     if (cmd == DCtty)
     {
@@ -532,7 +536,7 @@ QString GdbDriver::makeCmdString(DbgCommand cmd, QString strArg1, QString strArg
     normalizeStringArg(strArg1);
     normalizeStringArg(strArg2);
 
-    SIZED_QString(cmdString, MAX_FMTLEN+strArg1.length()+strArg2.length());
+    QString cmdString;
     cmdString.sprintf(cmds[cmd].fmt, strArg1.latin1(), strArg2.latin1());
     return cmdString;
 }
@@ -542,7 +546,7 @@ QString GdbDriver::makeCmdString(DbgCommand cmd, int intArg1, int intArg2)
     assert(cmd >= 0 && cmd < NUM_CMDS);
     assert(cmds[cmd].argsNeeded == GdbCmdInfo::argNum2);
 
-    SIZED_QString(cmdString, MAX_FMTLEN+60);
+    QString cmdString;
     cmdString.sprintf(cmds[cmd].fmt, intArg1, intArg2);
     return cmdString;
 }
@@ -654,12 +658,12 @@ static bool isErrorExpr(const char* output)
 
 /**
  * Returns true if the output is an error message. If wantErrorValue is
- * true, a new VarTree object is created and filled with the error message.
+ * true, a new ExprValue object is created and filled with the error message.
  * If there are warnings, they are skipped and output points past the warnings
  * on return (even if there \e are errors).
  */
 static bool parseErrorMessage(const char*& output,
-			      VarTree*& variable, bool wantErrorValue)
+			      ExprValue*& variable, bool wantErrorValue)
 {
     // skip warnings
     while (strncmp(output, "warning:", 8) == 0)
@@ -675,11 +679,11 @@ static bool parseErrorMessage(const char*& output,
     {
 	if (wantErrorValue) {
 	    // put the error message as value in the variable
-	    variable = new VarTree(QString(), VarTree::NKplain);
+	    variable = new ExprValue(QString(), VarTree::NKplain);
 	    const char* endMsg = strchr(output, '\n');
 	    if (endMsg == 0)
 		endMsg = output + strlen(output);
-	    variable->m_value = FROM_LATIN1(output, endMsg-output);
+	    variable->m_value = QString::fromLatin1(output, endMsg-output);
 	} else {
 	    variable = 0;
 	}
@@ -707,9 +711,9 @@ void GdbDriver::setPrintQStringDataCmd(const char* cmd)
     cmds[DCprintQStringStruct].fmt = cmd;
 }
 
-VarTree* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, bool qt3like)
+ExprValue* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, bool qt3like)
 {
-    VarTree* variable = 0;
+    ExprValue* variable = 0;
 
     /*
      * Parse off white space. gdb sometimes prints white space first if the
@@ -721,7 +725,7 @@ VarTree* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, boo
     // special case: empty string (0 repetitions)
     if (strncmp(output, "Invalid number 0 of repetitions", 31) == 0)
     {
-	variable = new VarTree(QString(), VarTree::NKplain);
+	variable = new ExprValue(QString(), VarTree::NKplain);
 	variable->m_value = "\"\"";
 	return variable;
     }
@@ -779,7 +783,7 @@ VarTree* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, boo
 		if (p == 0)
 		    goto error;
 		p++;			/* skip '>' */
-		repeatCount = FROM_LATIN1(start, p-start);
+		repeatCount = QString::fromLatin1(start, p-start);
 		while (isspace(*p) || *p == ',')
 		    p++;
 	    }
@@ -839,17 +843,17 @@ VarTree* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, boo
 	    result += "\"";
 
 	// assign the value
-	variable = new VarTree(QString(), VarTree::NKplain);
+	variable = new ExprValue(QString(), VarTree::NKplain);
 	variable->m_value = result;
     }
     else if (strncmp(p, "true", 4) == 0)
     {
-	variable = new VarTree(QString(), VarTree::NKplain);
+	variable = new ExprValue(QString(), VarTree::NKplain);
 	variable->m_value = "QString::null";
     }
     else if (strncmp(p, "false", 5) == 0)
     {
-	variable = new VarTree(QString(), VarTree::NKplain);
+	variable = new ExprValue(QString(), VarTree::NKplain);
 	variable->m_value = "(null)";
     }
     else
@@ -858,13 +862,13 @@ VarTree* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, boo
 
 error:
     if (wantErrorValue) {
-	variable = new VarTree(QString(), VarTree::NKplain);
+	variable = new ExprValue(QString(), VarTree::NKplain);
 	variable->m_value = "internal parse error";
     }
     return variable;
 }
 
-static VarTree* parseVar(const char*& s)
+static ExprValue* parseVar(const char*& s)
 {
     const char* p = s;
     
@@ -874,24 +878,41 @@ static VarTree* parseVar(const char*& s)
 
     QString name;
     VarTree::NameKind kind;
-    if (!parseName(p, name, kind)) {
-	return 0;
+    /*
+     * Detect anonymouse struct values: The 'name =' part is missing:
+     *    s = { a = 1, { b = 2 }}
+     * Note that this detection works only inside structs when the anonymous
+     * struct is not the first member:
+     *    s = {{ a = 1 }, b = 2}
+     * This is misparsed (by parseNested()) because it is mistakenly
+     * interprets the second opening brace as the first element of an array
+     * of structs.
+     */
+    if (*p == '{')
+    {
+	name = i18n("<anonymous struct or union>");
+	kind = VarTree::NKanonymous;
     }
-    
-    // go for '='
-    while (isspace(*p))
-	p++;
-    if (*p != '=') {
-	TRACE(QString().sprintf("parse error: = not found after %s", (const char*)name));
-	return 0;
-    }
-    // skip the '=' and more whitespace
-    p++;
-    while (isspace(*p))
-	p++;
+    else
+    {
+	if (!parseName(p, name, kind)) {
+	    return 0;
+	}
 
-    VarTree* variable = new VarTree(name, kind);
-    variable->setDeleteChildren(true);
+	// go for '='
+	while (isspace(*p))
+	    p++;
+	if (*p != '=') {
+	    TRACE(QString().sprintf("parse error: = not found after %s", (const char*)name));
+	    return 0;
+	}
+	// skip the '=' and more whitespace
+	p++;
+	while (isspace(*p))
+	    p++;
+    }
+
+    ExprValue* variable = new ExprValue(name, kind);
     
     if (!parseValue(p, variable)) {
 	delete variable;
@@ -1092,7 +1113,7 @@ static bool parseName(const char*& s, QString& name, VarTree::NameKind& kind)
 
     if (*p == '<') {
 	skipNestedAngles(p);
-	name = FROM_LATIN1(s, p - s);
+	name = QString::fromLatin1(s, p - s);
 	kind = VarTree::NKtype;
     }
     else
@@ -1118,14 +1139,14 @@ static bool parseName(const char*& s, QString& name, VarTree::NameKind& kind)
 	    }
 	    len = p - s;
 	}
-	name = FROM_LATIN1(s, len);
+	name = QString::fromLatin1(s, len);
     }
     // return the new position
     s = p;
     return true;
 }
 
-static bool parseValue(const char*& s, VarTree* variable)
+static bool parseValue(const char*& s, ExprValue* variable)
 {
     variable->m_value = "";
 
@@ -1141,7 +1162,7 @@ repeat:
 	{
 	    const char* start = s;
 	    skipNested(s, '{', '}');
-	    variable->m_value = FROM_LATIN1(start, s-start);
+	    variable->m_value = QString::fromLatin1(start, s-start);
 	    variable->m_value += ' ';	// add only a single space
 	    while (isspace(*s))
 		s++;
@@ -1155,7 +1176,7 @@ repeat:
 	    }
 	    // must be the closing brace
 	    if (*s != '}') {
-		TRACE("parse error: missing } of " +  variable->getText());
+		TRACE("parse error: missing } of " +  variable->m_name);
 		return false;
 	    }
 	    s++;
@@ -1192,7 +1213,7 @@ repeat:
 
 	    while (isspace(*p))
 		p++;
-	    variable->m_value = FROM_LATIN1(s, p - s);
+	    variable->m_value = QString::fromLatin1(s, p - s);
 	}
 
 	bool reference = false;
@@ -1300,7 +1321,7 @@ repeat:
 	    if (*p == '(')
 		skipNested(p, '(', ')');
 	}
-	variable->m_value += FROM_LATIN1(start, p - start);
+	variable->m_value += QString::fromLatin1(start, p - start);
 
 	// remove line breaks from the value; this is ok since
 	// string values never contain a literal line break
@@ -1324,7 +1345,7 @@ repeat:
 	    if (p != start) {
 		// there is always a blank before the string,
 		// which we will include in the final string value
-		variable->m_value += FROM_LATIN1(start-1, (p - start)+1);
+		variable->m_value += QString::fromLatin1(start-1, (p - start)+1);
 		// if this was a pointer, reset that flag since we 
 		// now got the value
 		variable->m_varKind = VarTree::VKsimple;
@@ -1332,7 +1353,7 @@ repeat:
 	}
 
 	if (variable->m_value.length() == 0) {
-	    TRACE("parse error: no value for " + variable->getText());
+	    TRACE("parse error: no value for " + variable->m_name);
 	    return false;
 	}
 
@@ -1348,16 +1369,12 @@ repeat:
 	if (reference) {
 	    goto repeat;
 	}
-
-	if (variable->m_varKind == VarTree::VKpointer) {
-	    variable->setDelayedExpanding(true);
-	}
     }
 
     return true;
 }
 
-static bool parseNested(const char*& s, VarTree* variable)
+static bool parseNested(const char*& s, ExprValue* variable)
 {
     // could be a structure or an array
     while (isspace(*s))
@@ -1398,10 +1415,10 @@ static bool parseNested(const char*& s, VarTree* variable)
     return true;
 }
 
-static bool parseVarSeq(const char*& s, VarTree* variable)
+static bool parseVarSeq(const char*& s, ExprValue* variable)
 {
     // parse a comma-separated sequence of variables
-    VarTree* var = variable;		/* var != 0 to indicate success if empty seq */
+    ExprValue* var = variable;		/* var != 0 to indicate success if empty seq */
     for (;;) {
 	if (*s == '}')
 	    break;
@@ -1425,7 +1442,7 @@ static bool parseVarSeq(const char*& s, VarTree* variable)
     return var != 0;
 }
 
-static bool parseValueSeq(const char*& s, VarTree* variable)
+static bool parseValueSeq(const char*& s, ExprValue* variable)
 {
     // parse a comma-separated sequence of variables
     int index = 0;
@@ -1433,8 +1450,7 @@ static bool parseValueSeq(const char*& s, VarTree* variable)
     for (;;) {
 	QString name;
 	name.sprintf("[%d]", index);
-	VarTree* var = new VarTree(name, VarTree::NKplain);
-	var->setDeleteChildren(true);
+	ExprValue* var = new ExprValue(name, VarTree::NKplain);
 	good = parseValue(s, var);
 	if (!good) {
 	    delete var;
@@ -1453,7 +1469,7 @@ static bool parseValueSeq(const char*& s, VarTree* variable)
 	    TRACE(QString().sprintf("found <repeats %d times> in array", l));
 	    // replace name and advance index
 	    name.sprintf("[%d .. %d]", index, index+l-1);
-	    var->setText(name);
+	    var->m_name = name;
 	    index += l;
 	    // skip " times>" and space
 	    s = end+7;
@@ -1467,8 +1483,7 @@ static bool parseValueSeq(const char*& s, VarTree* variable)
 	// long arrays may be terminated by '...'
 	if (strncmp(s, "...", 3) == 0) {
 	    s += 3;
-	    VarTree* var = new VarTree("...", VarTree::NKplain);
-	    var->setDeleteChildren(true);
+	    ExprValue* var = new ExprValue("...", VarTree::NKplain);
 	    var->m_value = i18n("<additional entries of the array suppressed>");
 	    variable->appendChild(var);
 	    break;
@@ -1503,7 +1518,7 @@ static void parseFrameInfo(const char*& s, QString& func,
 	    p++;
 	while (isxdigit(*p))
 	    p++;
-	address = FROM_LATIN1(start, p-start);
+	address = QString::fromLatin1(start, p-start);
 	if (strncmp(p, " in ", 4) == 0)
 	    p += 4;
     } else {
@@ -1512,7 +1527,7 @@ static void parseFrameInfo(const char*& s, QString& func,
     const char* start = p;
     // check for special signal handler frame
     if (strncmp(p, "<signal handler called>", 23) == 0) {
-	func = FROM_LATIN1(start, 23);
+	func = QString::fromLatin1(start, 23);
 	file = QString();
 	lineNo = -1;
 	s = p+23;
@@ -1524,7 +1539,8 @@ static void parseFrameInfo(const char*& s, QString& func,
     /*
      * Skip the function name. It is terminated by a left parenthesis
      * which does not delimit "(anonymous namespace)" and which is
-     * outside the angle brackets <> of template parameter lists.
+     * outside the angle brackets <> of template parameter lists
+     * and is preceded by a space.
      */
     while (*p != '\0')
     {
@@ -1542,9 +1558,14 @@ static void parseFrameInfo(const char*& s, QString& func,
 		skipNestedAngles(p);
 	    }
 	} else if (*p == '(') {
-	    if (strncmp(p, "(anonymous namespace)", 21) != 0)
-		break;	// parameter list found
-	    p += 21;
+	    // this skips "(anonymous namespace)" as well as the formal
+	    // parameter list of the containing function if this is a member
+	    // of a nested class
+	    skipNestedWithString(p, '(', ')');
+	} else if (*p == ' ') {
+	    ++p;
+	    if (*p == '(')
+		break; // parameter list found
 	} else {
 	    p++;
 	}
@@ -1589,7 +1610,7 @@ static void parseFrameInfo(const char*& s, QString& func,
 	do {
 	    --colon;
 	} while (*colon != ':');
-	file = FROM_LATIN1(fileStart, colon-fileStart);
+	file = QString::fromLatin1(fileStart, colon-fileStart);
 	lineNo = atoi(colon+1)-1;
 	// skip new-line
 	if (*p != '\0')
@@ -1612,7 +1633,7 @@ static void parseFrameInfo(const char*& s, QString& func,
     if (*p == '\0') {
 	func = start;
     } else {
-	func = FROM_LATIN1(start, p-start-1);	/* don't include \n */
+	func = QString::fromLatin1(start, p-start-1);	/* don't include \n */
     }
     s = p;
 
@@ -1689,7 +1710,7 @@ void GdbDriver::parseBackTrace(const char* output, QList<StackFrame>& stack)
 	frm->fileName = file;
 	frm->lineNo = lineNo;
 	frm->address = address;
-	frm->var = new VarTree(func, VarTree::NKplain);
+	frm->var = new ExprValue(func, VarTree::NKplain);
 	stack.append(frm);
     }
 }
@@ -1764,7 +1785,7 @@ bool GdbDriver::parseBreakList(const char* output, QList<Breakpoint>& brks)
 	    const char* start = p;
 	    while (*p != '\0' && !isspace(*p))
 		p++;
-	    address = FROM_LATIN1(start, p-start);
+	    address = QString::fromLatin1(start, p-start);
 	    while (isspace(*p) && *p != '\n')
 		p++;
 	    if (*p == '\0')
@@ -1781,7 +1802,7 @@ bool GdbDriver::parseBreakList(const char* output, QList<Breakpoint>& brks)
 	    location = p;
 	    p += location.length();
 	} else {
-	    location = FROM_LATIN1(p, end-p).stripWhiteSpace();
+	    location = QString::fromLatin1(p, end-p).stripWhiteSpace();
 	    p = end+1;			/* skip over \n */
 	}
 
@@ -1804,7 +1825,7 @@ bool GdbDriver::parseBreakList(const char* output, QList<Breakpoint>& brks)
 	    } else if (strncmp(p, "stop only if ", 13) == 0) {
 		// extract condition
 		p += 13;
-		condition = FROM_LATIN1(p, end-p).stripWhiteSpace();
+		condition = QString::fromLatin1(p, end-p).stripWhiteSpace();
 		TRACE("condition: "+condition);
 	    } else if (strncmp(p, "ignore next ", 12) == 0) {
 		// extract ignore count
@@ -1813,7 +1834,7 @@ bool GdbDriver::parseBreakList(const char* output, QList<Breakpoint>& brks)
 		TRACE(QString().sprintf("ignore count %d", ignoreCount));
 	    } else {
 		// indeed a continuation
-		location += " " + FROM_LATIN1(p, end-p).stripWhiteSpace();
+		location += " " + QString::fromLatin1(p, end-p).stripWhiteSpace();
 	    }
 	    p = end;
 	    if (*p != '\0')
@@ -1878,7 +1899,7 @@ bool GdbDriver::parseThreadList(const char* output, QList<ThreadInfo>& threads)
 	    // syntax error; bail out
 	    return true;
 	}
-	systag = FROM_LATIN1(p, end-p);
+	systag = QString::fromLatin1(p, end-p);
 	p = end+2;
 
 	/*
@@ -1953,7 +1974,7 @@ static bool parseNewBreakpoint(const char* o, int& id,
 	p += 6;
 	while (isxdigit(*p))
 	    ++p;
-	address = FROM_LATIN1(start, p-start);
+	address = QString::fromLatin1(start, p-start);
     }
     
     // file name
@@ -1964,7 +1985,7 @@ static bool parseNewBreakpoint(const char* o, int& id,
     
     // line number
     char* numStart = strstr(fileStart, ", line ");
-    QString fileName = FROM_LATIN1(fileStart, numStart-fileStart);
+    QString fileName = QString::fromLatin1(fileStart, numStart-fileStart);
     numStart += 7;
     int line = strtoul(numStart, &p, 10);
     if (numStart == p)
@@ -1989,11 +2010,11 @@ static bool parseNewWatchpoint(const char* o, int& id,
     p += 2;
 
     // all the rest on the line is the expression
-    expr = FROM_LATIN1(p, strlen(p)).stripWhiteSpace();
+    expr = QString::fromLatin1(p, strlen(p)).stripWhiteSpace();
     return true;
 }
 
-void GdbDriver::parseLocals(const char* output, QList<VarTree>& newVars)
+void GdbDriver::parseLocals(const char* output, QList<ExprValue>& newVars)
 {
     // check for possible error conditions
     if (strncmp(output, "No symbol table", 15) == 0)
@@ -2017,13 +2038,13 @@ void GdbDriver::parseLocals(const char* output, QList<VarTree>& newVars)
 	    continue;
 	}
 
-	VarTree* variable = parseVar(output);
+	ExprValue* variable = parseVar(output);
 	if (variable == 0) {
 	    break;
 	}
 	// do not add duplicates
-	for (VarTree* o = newVars.first(); o != 0; o = newVars.next()) {
-	    if (o->getText() == variable->getText()) {
+	for (ExprValue* o = newVars.first(); o != 0; o = newVars.next()) {
+	    if (o->m_name == variable->m_name) {
 		delete variable;
 		goto skipDuplicate;
 	    }
@@ -2033,17 +2054,16 @@ void GdbDriver::parseLocals(const char* output, QList<VarTree>& newVars)
     }
 }
 
-bool GdbDriver::parsePrintExpr(const char* output, bool wantErrorValue, VarTree*& var)
+ExprValue* GdbDriver::parsePrintExpr(const char* output, bool wantErrorValue)
 {
+    ExprValue* var = 0;
     // check for error conditions
-    if (parseErrorMessage(output, var, wantErrorValue))
+    if (!parseErrorMessage(output, var, wantErrorValue))
     {
-	return false;
-    } else {
 	// parse the variable
 	var = parseVar(output);
-	return true;
     }
+    return var;
 }
 
 bool GdbDriver::parseChangeWD(const char* output, QString& message)
@@ -2128,7 +2148,7 @@ uint GdbDriver::parseProgramStopped(const char* output, QString& message)
 	    const char* endOfMessage = strchr(start, '\n');
 	    if (endOfMessage == 0)
 		endOfMessage = start + strlen(start);
-	    message = FROM_LATIN1(start, endOfMessage-start);
+	    message = QString::fromLatin1(start, endOfMessage-start);
 	} else if (strncmp(start, "Breakpoint ", 11) == 0) {
 	    /*
 	     * We stopped at a (permanent) breakpoint (gdb doesn't tell us
@@ -2184,7 +2204,7 @@ void GdbDriver::parseSharedLibs(const char* output, QStrList& shlibs)
 	output = strchr(output, '\n');
 	if (output == 0)
 	    output = start + strlen(start);
-	shlibName = FROM_LATIN1(start, output-start);
+	shlibName = QString::fromLatin1(start, output-start);
 	if (*output != '\0')
 	    output++;
 	shlibs.append(shlibName);
@@ -2234,7 +2254,7 @@ void GdbDriver::parseRegisters(const char* output, QList<RegisterInfo>& regs)
 	    output++;
 	if (*output == '\0')
 	    break;
-	regName = FROM_LATIN1(start, output-start);
+	regName = QString::fromLatin1(start, output-start);
 
 	// skip space
 	while (isspace(*output))
@@ -2251,7 +2271,7 @@ void GdbDriver::parseRegisters(const char* output, QList<RegisterInfo>& regs)
 	{
 	    start = output;
 	    skipNested(output, '{', '}');
-	    value = FROM_LATIN1(start, output-start).simplifyWhiteSpace();
+	    value = QString::fromLatin1(start, output-start).simplifyWhiteSpace();
 	    // skip space, but not the end of line
 	    while (isspace(*output) && *output != '\n')
 		output++;
@@ -2268,7 +2288,7 @@ void GdbDriver::parseRegisters(const char* output, QList<RegisterInfo>& regs)
 
 		start = output;
 		skipNested(output, '{', '}');
-		value = FROM_LATIN1(start, output-start).simplifyWhiteSpace();
+		value = QString::fromLatin1(start, output-start).simplifyWhiteSpace();
 	    } else {
 		// for gdb 5.3
 		// find first type that does not have an array, this is the RAW value
@@ -2285,7 +2305,7 @@ void GdbDriver::parseRegisters(const char* output, QList<RegisterInfo>& regs)
 			end=cur;
 			while (*end && (*end!='}') && (*end!=',') && (*end!='\n'))
 			    end++;
-			QString rawValue = FROM_LATIN1(cur, end-cur).simplifyWhiteSpace();
+			QString rawValue = QString::fromLatin1(cur, end-cur).simplifyWhiteSpace();
 			reg->rawValue = rawValue;
 
 			if (rawValue.left(2)=="0x") {
@@ -2297,7 +2317,7 @@ void GdbDriver::parseRegisters(const char* output, QList<RegisterInfo>& regs)
 			    while (*cur!='{' && *cur!=' ')
 				cur--;
 			    cur++;
-			    reg->type=FROM_LATIN1(cur, end-cur);
+			    reg->type=QString::fromLatin1(cur, end-cur);
 			}
 
 			// end while loop 
@@ -2322,7 +2342,7 @@ void GdbDriver::parseRegisters(const char* output, QList<RegisterInfo>& regs)
 	    output = strchr(output,'\n');
 	    if (output == 0)
 		output = start + strlen(start);
-	    value = FROM_LATIN1(start, output-start).simplifyWhiteSpace();
+	    value = QString::fromLatin1(start, output-start).simplifyWhiteSpace();
 
 	    /*
 	     * We split the raw from the cooked values.
@@ -2373,7 +2393,7 @@ bool GdbDriver::parseInfoLine(const char* output, QString& addrFrom, QString& ad
     const char* p = start;
     while (*p != '\0' && !isspace(*p))
 	p++;
-    addrFrom = FROM_LATIN1(start, p-start);
+    addrFrom = QString::fromLatin1(start, p-start);
 
     start = strstr(p, "and ends at ");
     if (start == 0) {
@@ -2385,7 +2405,7 @@ bool GdbDriver::parseInfoLine(const char* output, QString& addrFrom, QString& ad
     p = start;
     while (*p != '\0' && !isspace(*p))
 	p++;
-    addrTo = FROM_LATIN1(start, p-start);
+    addrTo = QString::fromLatin1(start, p-start);
 
     return true;
 }
@@ -2423,7 +2443,7 @@ void GdbDriver::parseDisassemble(const char* output, QList<DisassembledCode>& co
 	// address
 	while (p != end && !isspace(*p))
 	    p++;
-	address = FROM_LATIN1(start, p-start);
+	address = QString::fromLatin1(start, p-start);
 
 	// function name (enclosed in '<>', followed by ':')
 	while (p != end && *p != '<')
@@ -2446,7 +2466,7 @@ void GdbDriver::parseDisassemble(const char* output, QList<DisassembledCode>& co
 
 	DisassembledCode* c = new DisassembledCode;
 	c->address = address;
-	c->code = FROM_LATIN1(start, p-start);
+	c->code = QString::fromLatin1(start, p-start);
 	code.append(c);
     }
 }
@@ -2468,7 +2488,7 @@ QString GdbDriver::parseMemoryDump(const char* output, QList<MemoryDump>& memdum
 	const char* start = p;
 	while (*p != '\0' && *p != ':' && !isspace(*p))
 	    p++;
-	addr = FROM_LATIN1(start, p-start);
+	addr = QString::fromLatin1(start, p-start);
 	if (*p != ':') {
 	    // parse function offset
 	    while (isspace(*p))
@@ -2476,7 +2496,7 @@ QString GdbDriver::parseMemoryDump(const char* output, QList<MemoryDump>& memdum
 	    start = p;
 	    while (*p != '\0' && !(*p == ':' && isspace(p[1])))
 		p++;
-	    addr.fnoffs = FROM_LATIN1(start, p-start);
+	    addr.fnoffs = QString::fromLatin1(start, p-start);
 	}
 	if (*p == ':')
 	    p++;
@@ -2486,10 +2506,10 @@ QString GdbDriver::parseMemoryDump(const char* output, QList<MemoryDump>& memdum
 	// everything to the end of the line is the memory dump
 	const char* end = strchr(p, '\n');
 	if (end != 0) {
-	    dump = FROM_LATIN1(p, end-p);
+	    dump = QString::fromLatin1(p, end-p);
 	    p = end+1;
 	} else {
-	    dump = FROM_LATIN1(p, strlen(p));
+	    dump = QString::fromLatin1(p, strlen(p));
 	    p += strlen(p);
 	}
 	MemoryDump* md = new MemoryDump;
@@ -2503,7 +2523,7 @@ QString GdbDriver::parseMemoryDump(const char* output, QList<MemoryDump>& memdum
 
 QString GdbDriver::editableValue(VarTree* value)
 {
-    const char* s = value->m_value.latin1();
+    const char* s = value->value().latin1();
 
     // if the variable is a pointer value that contains a cast,
     // remove the cast
@@ -2539,10 +2559,10 @@ repeat:
 	    ++s;
 	if (*s == '"') {
 	    // a string
-	    return FROM_LATIN1(start, end-start);
+	    return QString::fromLatin1(start, end-start);
 	} else {
 	    // other pointer
-	    return FROM_LATIN1(start, strlen(start));
+	    return QString::fromLatin1(start, strlen(start));
 	}
     }
 
