@@ -11,6 +11,7 @@
 #include <kfiledialog.h>
 #include <qtabdialog.h>
 #include <qfile.h>
+#include <qdragobject.h>
 #include "mainwndbase.h"
 #include "debugger.h"
 #include "gdbdriver.h"
@@ -41,7 +42,7 @@ WatchWindow::WatchWindow(QWidget* parent, const char* name, WFlags f) :
 	m_watchEdit(this, "watch_edit"),
 	m_watchAdd(i18n(" Add "), this, "watch_add"),
 	m_watchDelete(i18n(" Del "), this, "watch_delete"),
-	m_watchVariables(this, "watch_variables"),
+	m_watchVariables(this, i18n("Expression"), "watch_variables"),
 	m_watchV(this, 0),
 	m_watchH(0)
 {
@@ -57,10 +58,11 @@ WatchWindow::WatchWindow(QWidget* parent, const char* name, WFlags f) :
     connect(&m_watchEdit, SIGNAL(returnPressed()), SIGNAL(addWatch()));
     connect(&m_watchAdd, SIGNAL(clicked()), SIGNAL(addWatch()));
     connect(&m_watchDelete, SIGNAL(clicked()), SIGNAL(deleteWatch()));
-    connect(&m_watchVariables, SIGNAL(highlighted(int)), SLOT(slotWatchHighlighted(int)));
+    connect(&m_watchVariables, SIGNAL(currentChanged(QListViewItem*)),
+	    SLOT(slotWatchHighlighted()));
 
-    m_watchVariables.setMoveCurrentToSibling(true);
     m_watchVariables.installEventFilter(this);
+    setAcceptDrops(true);
 }
 
 WatchWindow::~WatchWindow()
@@ -80,11 +82,33 @@ bool WatchWindow::eventFilter(QObject*, QEvent* ev)
     return false;
 }
 
+void WatchWindow::dragEnterEvent(QDragEnterEvent* event)
+{
+    event->accept(QTextDrag::canDecode(event));
+}
+
+void WatchWindow::dropEvent(QDropEvent* event)
+{
+    QString text;
+    if (QTextDrag::decode(event, text))
+    {
+	// pick only the first line
+	text = text.stripWhiteSpace();
+	int pos = text.find('\n');
+	if (pos > 0)
+	    text.truncate(pos);
+	text = text.stripWhiteSpace();
+	if (!text.isEmpty())
+	    emit textDropped(text);
+    }
+}
+
 
 // place the text of the hightlighted watch expr in the edit field
-void WatchWindow::slotWatchHighlighted(int idx)
+void WatchWindow::slotWatchHighlighted()
 {
-    QString text = m_watchVariables.exprStringAt(idx);
+    VarTree* expr = m_watchVariables.selectedItem();
+    QString text = expr ? expr->computeExpr() : QString();
     m_watchEdit.setText(text);
 }
 
