@@ -30,6 +30,8 @@ public:
     ~WatchWindow();
     ExprWnd* watchVariables() { return &m_watchVariables; }
     QString watchText() const { return m_watchEdit.text(); }
+    int columnWidth(int i) const { return m_watchVariables.columnWidth(i); }
+    void setColumnWidth(int i, int w) { m_watchVariables.setColumnWidth(i, w); }
 
 protected:
     QLineEdit m_watchEdit;
@@ -40,13 +42,16 @@ protected:
     QHBoxLayout m_watchH;
 
     virtual bool eventFilter(QObject* ob, QEvent* ev);
+    virtual void dragEnterEvent(QDragEnterEvent* event);
+    virtual void dropEvent(QDropEvent* event);
 
 signals:
     void addWatch();
     void deleteWatch();
+    void textDropped(const QString& text);
 
 protected slots:
-    void slotWatchHighlighted(int);
+    void slotWatchHighlighted();
 };
 
 
