@@ -10,9 +10,12 @@
 #include <qbrush.h>
 #include <qfile.h>
 #include <qkeycode.h>
+#include <qpopupmenu.h>
 #include <kapp.h>
 #include <kiconloader.h>
 #include <kglobalsettings.h>
+#include <kmainwindow.h>
+#include <algorithm>
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -20,11 +23,12 @@
 
 
 SourceWindow::SourceWindow(const char* fileName, QWidget* parent, const char* name) :
-	KTextView(parent, name),
-	m_fileName(fileName)
+	QTextEdit(parent, name),
+	m_fileName(fileName),
+	m_curRow(-1),
+	m_widthItems(16),
+	m_widthPlus(12)
 {
-    setNumCols(3);
-
     // load pixmaps
     m_pcinner = UserIcon("pcinner");
     m_pcup = UserIcon("pcup");
@@ -34,45 +38,49 @@ SourceWindow::SourceWindow(const char* fileName, QWidget* parent, const char* na
     m_brkcond = UserIcon("brkcond");
     m_brkorph = UserIcon("brkorph");
     setFont(KGlobalSettings::fixedFont());
+    setReadOnly(true);
+    setMargins(m_widthItems+m_widthPlus, 0, 0 ,0);
+    setAutoFormatting(AutoNone);
+    setTextFormat(PlainText);
+    setWordWrap(NoWrap);
+    connect(verticalScrollBar(), SIGNAL(valueChanged(int)),
+	    this, SLOT(update()));
+    connect(this, SIGNAL(cursorPositionChanged(int,int)), this, SLOT(cursorChanged(int)));
+    viewport()->installEventFilter(this);
+
+    // add a syntax highlighter
+    if (QRegExp("\\.(c(pp|c|\\+\\+)?|CC?|h(\\+\\+|h)?|HH?)$").search(m_fileName))
+    {
+	new HighlightCpp(this);
+    }
 }
 
 SourceWindow::~SourceWindow()
 {
+    delete syntaxHighlighter();
 }
 
 bool SourceWindow::loadFile()
 {
-    // first we load the code into KTextView::m_texts
+    // first we load the code into QTextEdit
     QFile f(m_fileName);
     if (!f.open(IO_ReadOnly)) {
 	return false;
     }
 
-    bool upd = autoUpdate();
-    setAutoUpdate(false);
-
     QTextStream t(&f);
-    QString s;
-    while (!t.eof()) {
-	s = t.readLine();
-	insertLine(s);
-    }
+    setText(t.read());
     f.close();
 
-    setAutoUpdate(upd);
-    if (upd) {
-	updateTableSize();
-    }
-
     // then we copy it into our own m_sourceCode
-    m_sourceCode.setSize(m_texts.size());
-    m_rowToLine.setSize(m_texts.size());
-    m_lineItems.setSize(m_texts.size());
-    for (int i = 0; i < m_texts.size(); i++) {
-	m_sourceCode[i].code = m_texts[i];
+    int n = paragraphs();
+    m_sourceCode.resize(n);
+    m_rowToLine.resize(n);
+    for (int i = 0; i < n; i++) {
+	m_sourceCode[i].code = text(i);
 	m_rowToLine[i] = i;
-	m_lineItems[i] = 0;
     }
+    m_lineItems.resize(n, 0);
 
     return true;
 }
@@ -86,65 +94,27 @@ void SourceWindow::reloadFile()
     }
 
     // read text into m_sourceCode
-    m_sourceCode.setSize(0);		/* clear old text */
+    m_sourceCode.clear();		/* clear old text */
 
     QTextStream t(&f);
-    SourceLine s;
-    while (!t.eof()) {
-	s.code = t.readLine();
-	m_sourceCode.append(s);
-    }
+    setText(t.read());
     f.close();
 
-    bool autoU = autoUpdate();
-    setAutoUpdate(false);
-
-    int lineNo = QMIN(m_texts.size(), m_sourceCode.size());
-    for (int i = 0; i < lineNo; i++) {
-	replaceLine(i, m_sourceCode[i].code);
+    m_sourceCode.resize(paragraphs());
+    for (size_t i = 0; i < m_sourceCode.size(); i++) {
+	m_sourceCode[i].code = text(i);
     }
-    if (m_sourceCode.size() > m_texts.size()) {
-	// the new file has more lines than the old one
-	// here lineNo is the number of lines of the old file
-	for (int i = lineNo; i < m_sourceCode.size(); i++) {
-	    insertLine(m_sourceCode[i].code);
-	}
-	// allocate line items
-	m_lineItems.setSize(m_texts.size());
-	for (int i = m_texts.size()-1; i >= lineNo; i--) {
-	    m_lineItems[i] = 0;
-	}
-    } else {
-	// the new file has fewer lines
-	// here lineNo is the number of lines of the new file
-	// remove the excessive lines
-	m_texts.setSize(lineNo);
-	m_lineItems.setSize(lineNo);
-    }
-    f.close();
+    // allocate line items
+    m_lineItems.resize(m_sourceCode.size(), 0);
 
-    setNumRows(m_texts.size());
-
-    m_rowToLine.setSize(m_texts.size());
-    for (int i = 0; i < m_texts.size(); i++)
+    m_rowToLine.resize(m_sourceCode.size());
+    for (size_t i = 0; i < m_sourceCode.size(); i++)
 	m_rowToLine[i] = i;
-
-    setAutoUpdate(autoU);
-    if (autoU && isVisible()) {
-	updateTableSize();
-	repaint();
-    }
-
-    // if the cursor is in the deleted lines, move it to the last line
-    if (m_curRow >= m_texts.size()) {
-	m_curRow = -1;			/* at this point don't have an active row */
-	activateLine(m_texts.size()-1);	/* now we have */
-    }
 }
 
 void SourceWindow::scrollTo(int lineNo, const DbgAddr& address)
 {
-    if (lineNo < 0 || lineNo >= m_sourceCode.size())
+    if (lineNo < 0 || lineNo >= int(m_sourceCode.size()))
 	return;
 
     int row = lineToRow(lineNo, address);
@@ -153,59 +123,43 @@ void SourceWindow::scrollTo(int lineNo, const DbgAddr& address)
 
 void SourceWindow::scrollToRow(int row)
 {
-    if (row >= numRows())
-	row = numRows();
-
-    // scroll to lineNo
-    if (row >= topCell() && row <= lastRowVisible()) {
-	// line is already visible
-	setCursorPosition(row, 0);
-	return;
-    }
-
-    // setCursorPosition does only a half-hearted job of making the
-    // cursor line visible, so we do it by hand...
     setCursorPosition(row, 0);
-    row -= 5;
-    if (row < 0)
-	row = 0;
-
-    setTopCell(row);
+    ensureCursorVisible();
 }
 
-int SourceWindow::textCol() const
+void SourceWindow::drawFrame(QPainter* p)
 {
-    // text is in column 2
-    return 2;
-}
+    QTextEdit::drawFrame(p);
 
-int SourceWindow::cellWidth(int col) const
-{
-    if (col == 0) {
-	return 15;
-    } else if (col == 1) {
-	return 12;
-    } else {
-	return KTextView::cellWidth(col);
-    }
-}
+    // and paragraph at the top is...
+    int top = paragraphAt(QPoint(0,contentsY()));
+    int bot = paragraphAt(QPoint(0,contentsY()+visibleHeight()-1));
+    if (bot < 0)
+	bot = paragraphs()-1;
 
-void SourceWindow::paintCell(QPainter* p, int row, int col)
-{
-    if (col == textCol()) {
-	p->save();
-	if (isRowDisassCode(row)) {
-	    p->setPen(blue);
-	}
-	KTextView::paintCell(p, row, col);
-	p->restore();
-	return;
-    }
-    if (col == 0) {
+    p->save();
+
+    // set a clip rectangle
+    int fw = frameWidth();
+    QRect inside = rect();
+    inside.addCoords(fw,fw,-fw,-fw);
+    QRegion clip = p->clipRegion();
+    clip &= QRegion(inside);
+    p->setClipRegion(clip);
+
+    p->setPen(colorGroup().text());
+    p->eraseRect(inside);
+
+    for (int row = top; row <= bot; row++)
+    {
 	uchar item = m_lineItems[row];
-	if (item == 0)			/* shortcut out */
-	    return;
-	int h = cellHeight(row);
+	p->save();
+
+	QRect r = paragraphRect(row);
+	QPoint pt = contentsToViewport(r.topLeft());
+	int h = r.height();
+	p->translate(fw, pt.y()+viewport()->y());
+
 	if (item & liBP) {
 	    // enabled breakpoint
 	    int y = (h - m_brkena.height())/2;
@@ -248,12 +202,9 @@ void SourceWindow::paintCell(QPainter* p, int row, int col)
 	    if (y < 0) y = 0;
 	    p->drawPixmap(0,y,m_pcup);
 	}
-	return;
-    }
-    if (col == 1) {
 	if (!isRowDisassCode(row) && m_sourceCode[rowToLine(row)].canDisass) {
-	    int h = cellHeight(row);
-	    int w = cellWidth(col);
+	    int w = m_widthPlus;
+	    p->translate(m_widthItems, 0);
 	    int x = w/2;
 	    int y = h/2;
 	    p->drawLine(x-2, y, x+2, y);
@@ -261,7 +212,9 @@ void SourceWindow::paintCell(QPainter* p, int row, int col)
 		p->drawLine(x, y-2, x, y+2);
 	    }
 	}
+	p->restore();
     }
+    p->restore();
 }
 
 void SourceWindow::updateLineItems(const KDebugger* dbg)
@@ -286,7 +239,7 @@ void SourceWindow::updateLineItems(const KDebugger* dbg)
 	    if (j < 0) {
 		/* doesn't exist anymore, remove it */
 		m_lineItems[i] &= ~liBPany;
-		updateLineItem(i);
+		update();
 	    }
 	}
     }
@@ -297,7 +250,7 @@ void SourceWindow::updateLineItems(const KDebugger* dbg)
 	if (fileNameMatches(bp->fileName)) {
 	    TRACE(QString().sprintf("updating %s:%d", bp->fileName.data(), bp->lineNo));
 	    int i = bp->lineNo;
-	    if (i < 0 || i >= m_sourceCode.size())
+	    if (i < 0 || i >= int(m_sourceCode.size()))
 		continue;
 	    // compute new line item flags for breakpoint
 	    uchar flags = bp->enabled ? liBP : liBPdisabled;
@@ -312,17 +265,12 @@ void SourceWindow::updateLineItems(const KDebugger* dbg)
 	    if ((m_lineItems[row] & liBPany) != flags) {
 		m_lineItems[row] &= ~liBPany;
 		m_lineItems[row] |= flags;
-		updateLineItem(row);
+		update();
 	    }
 	}
     }
 }
 
-void SourceWindow::updateLineItem(int row)
-{
-    updateCell(row, 0);
-}
-
 void SourceWindow::setPC(bool set, int lineNo, const DbgAddr& address, int frameNo)
 {
     if (lineNo < 0 || lineNo >= int(m_sourceCode.size())) {
@@ -336,13 +284,13 @@ void SourceWindow::setPC(bool set, int lineNo, const DbgAddr& address, int frame
 	// set only if not already set
 	if ((m_lineItems[row] & flag) == 0) {
 	    m_lineItems[row] |= flag;
-	    updateLineItem(row);
+	    update();
 	}
     } else {
 	// clear only if not set
 	if ((m_lineItems[row] & flag) != 0) {
 	    m_lineItems[row] &= ~flag;
-	    updateLineItem(row);
+	    update();
 	}
     }
 }
@@ -350,53 +298,30 @@ void SourceWindow::setPC(bool set, int lineNo, const DbgAddr& address, int frame
 void SourceWindow::find(const QString& text, bool caseSensitive, FindDirection dir)
 {
     ASSERT(dir == 1 || dir == -1);
-    if (m_sourceCode.size() == 0 || text.isEmpty())
+    if (QTextEdit::find(text, caseSensitive, false, dir > 0))
 	return;
-
-    int line;
-    DbgAddr dummyAddr;
-    activeLine(line, dummyAddr);
-    if (line < 0)
-	line = 0;
-    int curLine = line;			/* remember where we started */
-    bool found = false;
-    do {
-	// advance and wrap around
-	line += dir;
-	if (line < 0)
-	    line = m_sourceCode.size()-1;
-	else if (line >= int(m_sourceCode.size()))
-	    line = 0;
-	// search text
-	found = m_sourceCode[line].code.find(text, 0, caseSensitive) >= 0;
-    } while (!found && line != curLine);
-
-    scrollTo(line, DbgAddr());
+    // not found; wrap around
+    int para = dir > 0 ? 0 : paragraphs(), index = 0;
+    QTextEdit::find(text, caseSensitive, false, dir > 0, &para, &index);
 }
 
 void SourceWindow::mousePressEvent(QMouseEvent* ev)
 {
-    // Check if right button was clicked.
-    if (ev->button() == RightButton)
+    // we handle left and middle button
+    if (ev->button() != LeftButton && ev->button() != MidButton)
     {
-	emit clickedRight(ev->pos());
-	return;
-    }
-
-    int col = findCol(ev->x());
-
-    // check if event is in line item column
-    if (col == textCol()) {
-	KTextView::mousePressEvent(ev);
+	QTextEdit::mousePressEvent(ev);
 	return;
     }
 
     // get row
-    int row = findRow(ev->y());
-    if (row < 0 || col < 0)
+    QPoint p = viewportToContents(QPoint(0, ev->y() - viewport()->y()));
+    int row = paragraphAt(p);
+    if (row < 0)
 	return;
 
-    if (col == 1) {
+    if (ev->x() > m_widthItems+frameWidth())
+    {
 	if (isRowExpanded(row)) {
 	    actionCollapseRow(row);
 	} else {
@@ -432,6 +357,8 @@ void SourceWindow::mousePressEvent(QMouseEvent* ev)
 
 void SourceWindow::keyPressEvent(QKeyEvent* ev)
 {
+    int top1, top2;
+    QPoint top;
     switch (ev->key()) {
     case Key_Plus:
 	actionExpandRow(m_curRow);
@@ -439,9 +366,37 @@ void SourceWindow::keyPressEvent(QKeyEvent* ev)
     case Key_Minus:
 	actionCollapseRow(m_curRow);
 	return;
+    case Key_Up:
+	if (m_curRow > 0) {
+	    setCursorPosition(m_curRow-1, 0);
+	}
+	return;
+    case Key_Down:
+	if (m_curRow < paragraphs()-1) {
+	    setCursorPosition(m_curRow+1, 0);
+	}
+	return;
+    case Key_Home:
+	setCursorPosition(0, 0);
+	return;
+    case Key_End:
+	setCursorPosition(paragraphs()-1, 0);
+	return;
+    case Key_Next:
+    case Key_Prior:
+	top = viewportToContents(QPoint(0,0));
+	top1 = paragraphAt(top);
     }
 
-    KTextView::keyPressEvent(ev);
+    QTextEdit::keyPressEvent(ev);
+
+    switch (ev->key()) {
+    case Key_Next:
+    case Key_Prior:
+	top = viewportToContents(QPoint(0,0));
+	top2 = paragraphAt(top);
+	setCursorPosition(m_curRow+(top2-top1), 0);
+    }
 }
 
 static inline bool isident(QChar c)
@@ -451,78 +406,33 @@ static inline bool isident(QChar c)
 
 bool SourceWindow::wordAtPoint(const QPoint& p, QString& word, QRect& r)
 {
-    if (findCol(p.x()) != textCol())
-	return false;
-    int row = findRow(p.y());
-    if (row < 0)
+    QPoint pv = viewportToContents(p - viewport()->pos());
+    int row, col  = charAt(pv, &row);
+    if (row < 0 || col < 0)
 	return false;
 
-    // find top-left corner of text cell
-    int top, left;
-    if (!colXPos(textCol(), &left))
-	return false;
-    if (!rowYPos(row, &top))
+    // isolate the word at row, col
+    QString line = text(row);
+    if (!isident(line[col]))
 	return false;
 
-    // get the bounding rect of the text
-    QPainter painter(this);
-    setupPainter(&painter);
-    const QString& text = m_texts[row];
-    QRect bound =
-	painter.boundingRect(left+2, top, 0,0,
-			     AlignLeft | SingleLine | DontClip | ExpandTabs,
-			     text, text.length());
-    if (!bound.contains(p))
-	return false;			/* p is outside text */
+    int begin = col;
+    while (begin > 0 && isident(line[begin-1]))
+	--begin;
+    do
+	++col;
+    while (col < int(line.length()) && isident(line[col]));
 
-    /*
-     * We split the line into words and check each whether it contains
-     * the point p. We must do it the hard way (measuring substrings)
-     * because we cannot rely on that the font is mono-spaced.
-     */
-    uint start = 0;
-    while (start < text.length()) {
-	while (start < text.length() && !isident(text[start]))
-	    start++;
-	if (start >= text.length())
-	    return false;
-	/*
-	 * If p is in the rectangle that ends at 'start', it is in the
-	 * last non-word part that we have just skipped.
-	 */
-	bound = 
-	    painter.boundingRect(left+2, top, 0,0,
-				 AlignLeft | SingleLine | DontClip | ExpandTabs,
-				 text, start);
-	if (bound.contains(p))
-	    return false;
-	// a word starts now
-	int startWidth = bound.width();
-	uint end = start;
-	while (end < text.length() && isident(text[end]))
-	    end++;
-	bound =
-	    painter.boundingRect(left+2, top, 0,0,
-				 AlignLeft | SingleLine | DontClip | ExpandTabs,
-				 text, end);
-	if (bound.contains(p)) {
-	    // we found a word!
-	    // extract the word
-	    word = text.mid(start, end-start);
-	    // and the rectangle
-	    r = QRect(bound.x()+startWidth,bound.y(),
-		      bound.width()-startWidth, bound.height());
-	    return true;
-	}
-	start = end;
-    }
-    return false;
+    r = QRect(p, p);
+    r.addCoords(-5,-5,5,5);
+    word = line.mid(begin, col-begin);
+    return true;
 }
 
 void SourceWindow::paletteChange(const QPalette& oldPal)
 {
     setFont(KGlobalSettings::fixedFont());
-    KTextView::paletteChange(oldPal);
+    QTextEdit::paletteChange(oldPal);
 }
 
 /*
@@ -550,19 +460,22 @@ bool SourceWindow::fileNameMatches(const QString& other)
 void SourceWindow::disassembled(int lineNo, const QList<DisassembledCode>& disass)
 {
     TRACE("disassembled line " + QString().setNum(lineNo));
-    if (lineNo < 0 || lineNo >= m_sourceCode.size())
+    if (lineNo < 0 || lineNo >= int(m_sourceCode.size()))
 	return;
 
     SourceLine& sl = m_sourceCode[lineNo];
 
     // copy disassembled code and its addresses
-    sl.disass.setSize(disass.count());
-    sl.disassAddr.setSize(disass.count());
+    sl.disass.resize(disass.count());
+    sl.disassAddr.resize(disass.count());
     sl.canDisass = disass.count() > 0;
     for (uint i = 0; i < disass.count(); i++) {
 	const DisassembledCode* c =
 	    const_cast<QList<DisassembledCode>&>(disass).at(i);
-	sl.disass[i] = c->address.asString() + ' ' + c->code;
+	QString code = c->code;
+	while (code.endsWith("\n"))
+	    code.truncate(code.length()-1);
+	sl.disass[i] = c->address.asString() + ' ' + code;
 	sl.disassAddr[i] = c->address;
     }
 
@@ -571,7 +484,7 @@ void SourceWindow::disassembled(int lineNo, const QList<DisassembledCode>& disas
 	expandRow(row);
     } else {
 	// clear expansion marker
-	updateCell(row, 1);
+	update();
     }
 }
 
@@ -602,7 +515,7 @@ int SourceWindow::lineToRow(int line)
 {
     // line is zero-based!
 
-    assert(line < m_rowToLine.size());
+    assert(line < int(m_rowToLine.size()));
 
     // quick test for common case
     if (line < 0 || m_rowToLine[line] == line)
@@ -619,7 +532,7 @@ int SourceWindow::lineToRow(int line)
     int h = m_rowToLine.size();
     while (l < h && m_rowToLine[l] != line)
     {
-	assert(h == m_rowToLine.size() || m_rowToLine[l] < m_rowToLine[h]);
+	assert(h == int(m_rowToLine.size()) || m_rowToLine[l] < m_rowToLine[h]);
 
 	/*
 	 * We want to round down the midpoint so that we find the
@@ -655,13 +568,13 @@ int SourceWindow::lineToRow(int line, const DbgAddr& address)
 bool SourceWindow::isRowExpanded(int row)
 {
     assert(row >= 0);
-    return row < m_rowToLine.size()-1 &&
+    return row < int(m_rowToLine.size())-1 &&
 	m_rowToLine[row] == m_rowToLine[row+1];
 }
 
 bool SourceWindow::isRowDisassCode(int row)
 {
-    return row > 0 &&
+    return row > 0 && row < int(m_rowToLine.size()) &&
 	m_rowToLine[row] == m_rowToLine[row-1];
 }
 
@@ -670,34 +583,30 @@ void SourceWindow::expandRow(int row)
     TRACE("expanding row " + QString().setNum(row));
     // get disassembled code
     int line = rowToLine(row);
-    const ValArray<QString>& disass = m_sourceCode[line].disass;
+    const std::vector<QString>& disass = m_sourceCode[line].disass;
 
     // remove PC (must be set again in slot of signal expanded())
     m_lineItems[row] &= ~(liPC|liPCup);
 
-    // insert new lines
-    ++row;
-    m_rowToLine.insertAt(row, line, disass.size());
-    m_lineItems.insertAt(row, 0, disass.size());
-    m_texts.insertAt(row, disass);
-
-    bool autoU = autoUpdate();
-    setAutoUpdate(false);
-
-    // update line widths
-    for (int i = 0; i < disass.size(); i++) {
-	updateCellSize(disass[i]);
+    // adjust current row
+    if (m_curRow > row) {
+	m_curRow += disass.size();
+	// highlight is moved automatically
     }
 
-    setNumRows(m_texts.size());
+    // insert new lines
+    setUpdatesEnabled(false);
+    ++row;
+    for (size_t i = 0; i < disass.size(); i++) {
+	m_rowToLine.insert(m_rowToLine.begin()+row, line);
+	m_lineItems.insert(m_lineItems.begin()+row, 0);
+	insertParagraph(disass[i], row++);
+    }
+    setUpdatesEnabled(true);
+    viewport()->update();
+    update();		// line items
 
     emit expanded(line);		/* must set PC */
-
-    setAutoUpdate(autoU);
-    if (autoU && isVisible()) {
-	updateTableSize();
-	update();
-    }
 }
 
 void SourceWindow::collapseRow(int row)
@@ -707,32 +616,32 @@ void SourceWindow::collapseRow(int row)
 
     // find end of this block
     int end = row+1;
-    while (end < m_rowToLine.size() && m_rowToLine[end] == m_rowToLine[row]) {
+    while (end < int(m_rowToLine.size()) && m_rowToLine[end] == m_rowToLine[row]) {
 	end++;
     }
     ++row;
-    m_rowToLine.removeAt(row, end-row);
-    m_lineItems.removeAt(row, end-row);
-    m_texts.removeAt(row, end-row);
-
-    bool autoU = autoUpdate();
-    setAutoUpdate(false);
-
-    setNumRows(m_texts.size());
+    // adjust current row
+    if (m_curRow >= row) {
+	m_curRow -= end-row;
+	if (m_curRow < row)	// was m_curRow in disassembled code?
+	    m_curRow = -1;
+    }
+    setUpdatesEnabled(false);
+    while (--end >= row) {
+	m_rowToLine.erase(m_rowToLine.begin()+end);
+	m_lineItems.erase(m_lineItems.begin()+end);
+	removeParagraph(end);
+    }
+    setUpdatesEnabled(true);
+    viewport()->update();
+    update();		// line items
 
     emit collapsed(line);
-
-    setAutoUpdate(autoU);
-    if (autoU && isVisible()) {
-	updateTableSize();
-	update();
-    }
 }
 
 void SourceWindow::activeLine(int& line, DbgAddr& address)
 {
-    int row, col;
-    cursorPosition(&row, &col);
+    int row = m_curRow;
 
     int sourceRow;
     line = rowToLine(row, &sourceRow);
@@ -752,7 +661,7 @@ int SourceWindow::SourceLine::findAddressRowOffset(const DbgAddr& address) const
     if (address.isEmpty())
 	return 0;
 
-    for (int i = 0; i < disassAddr.size(); i++) {
+    for (size_t i = 0; i < disassAddr.size(); i++) {
 	if (disassAddr[i] == address) {
 	    // found exact address
 	    return i+1;
@@ -773,10 +682,8 @@ int SourceWindow::SourceLine::findAddressRowOffset(const DbgAddr& address) const
 
 void SourceWindow::actionExpandRow(int row)
 {
-    if (isRowExpanded(row) || isRowDisassCode(row)) {
-	// do nothing
+    if (row < 0 || isRowExpanded(row) || isRowDisassCode(row))
 	return;
-    }
 
     // disassemble
     int line = rowToLine(row);
@@ -792,13 +699,257 @@ void SourceWindow::actionExpandRow(int row)
 
 void SourceWindow::actionCollapseRow(int row)
 {
-    if (!isRowExpanded(row) || isRowDisassCode(row)) {
-	// do nothing
+    if (row < 0 || !isRowExpanded(row) || isRowDisassCode(row))
 	return;
-    }
 
     collapseRow(row);
 }
 
+void SourceWindow::setTabWidth(int numChars)
+{
+    if (numChars <= 0)
+	numChars = 8;
+    QFontMetrics fm(currentFont());
+    QString s;
+    int w = fm.width(s.fill('x', numChars));
+    setTabStopWidth(w);
+}
+
+void SourceWindow::cursorChanged(int row)
+{
+    if (row == m_curRow)
+	return;
+
+    if (m_curRow >= 0 && m_curRow < paragraphs())
+	clearParagraphBackground(m_curRow);
+    m_curRow = row;
+    setParagraphBackgroundColor(row, colorGroup().background());
+
+    emit lineChanged();
+}
+
+/*
+ * We must override the context menu handling because QTextEdit's handling
+ * requires that it receives ownership of the popup menu; but the popup menu
+ * returned from the GUI factory is owned by the factory.
+ */
+
+void SourceWindow::contextMenuEvent(QContextMenuEvent* e)
+{
+    // get the context menu from the GUI factory
+    QWidget* top = this;
+    do
+	top = top->parentWidget();
+    while (!top->isTopLevel());
+    KMainWindow* mw = static_cast<KMainWindow*>(top);
+    QPopupMenu* m =
+	static_cast<QPopupMenu*>(mw->factory()->container("popup_files", mw));
+    m->exec(e->globalPos());
+}
+
+bool SourceWindow::eventFilter(QObject* watched, QEvent* e)
+{
+    if (e->type() == QEvent::ContextMenu && watched == viewport())
+    {
+	contextMenuEvent(static_cast<QContextMenuEvent*>(e));
+	return true;
+    }
+    return QTextEdit::eventFilter(watched, e);
+}
+
+HighlightCpp::HighlightCpp(SourceWindow* srcWnd) :
+	QSyntaxHighlighter(srcWnd),
+	m_srcWnd(srcWnd)
+{
+}
+
+enum HLState {
+    hlCommentLine = 1,
+    hlCommentBlock,
+    hlIdent,
+    hlString
+};
+
+static const QString ckw[] =
+{
+    "and",
+    "and_eq",
+    "asm",
+    "auto",
+    "bitand",
+    "bitor",
+    "bool",
+    "break",
+    "case",
+    "catch",
+    "char",
+    "class",
+    "compl",
+    "const",
+    "const_cast",
+    "continue",
+    "default",
+    "delete",
+    "do",
+    "double",
+    "dynamic_cast",
+    "else",
+    "enum",
+    "explicit",
+    "export",
+    "extern",
+    "false",
+    "float",
+    "for",
+    "friend",
+    "goto",
+    "if",
+    "inline",
+    "int",
+    "long",
+    "mutable",
+    "namespace",
+    "new",
+    "not",
+    "not_eq",
+    "operator",
+    "or",
+    "or_eq",
+    "private",
+    "protected",
+    "public",
+    "reinterpret_cast",
+    "register",
+    "return",
+    "short",
+    "signed",
+    "sizeof",
+    "static",
+    "static_cast",
+    "struct",
+    "switch",
+    "template",
+    "this",
+    "throw",
+    "true",
+    "try",
+    "typedef",
+    "typeid",
+    "typename",
+    "using",
+    "union",
+    "unsigned",
+    "virtual",
+    "void",
+    "volatile",
+    "wchar_t",
+    "while",
+    "xor",
+    "xor_eq"
+};
+
+int HighlightCpp::highlightParagraph(const QString& text, int state)
+{
+    int row = currentParagraph();
+    // highlight assembly lines
+    if (m_srcWnd->isRowDisassCode(row))
+    {
+	setFormat(0, text.length(), blue);
+	return state;
+    }
+
+    if (state == -2)		// initial state
+	state = 0;
+
+    // check for preprocessor line
+    if (state == 0 && text.stripWhiteSpace().startsWith("#"))
+    {
+	setFormat(0, text.length(), QColor("dark green"));
+	return 0;
+    }
+
+    // a font for keywords
+    QFont identFont = textEdit()->currentFont();
+    identFont.setBold(!identFont.bold());
+
+    unsigned start = 0;
+    while (start < text.length())
+    {
+	int end;
+	switch (state) {
+	case hlCommentLine:
+	    end = text.length();
+	    state = 0;
+	    setFormat(start, end-start, QColor("gray50"));
+	    break;
+	case hlCommentBlock:
+	    end = text.find("*/", start);
+	    if (end >= 0)
+		end += 2, state = 0;
+	    else
+		end = text.length();
+	    setFormat(start, end-start, QColor("gray50"));
+	    break;
+	case hlString:
+	    for (end = start+1; end < int(text.length()); end++) {
+		if (text[end] == '\\') {
+		    if (end < int(text.length()))
+			++end;
+		} else if (text[end] == text[start]) {
+		    ++end;
+		    break;
+		}
+	    }
+	    state = 0;
+	    setFormat(start, end-start, QColor("dark red"));
+	    break;
+	case hlIdent:
+	    for (end = start+1; end < int(text.length()); end++) {
+		if (!text[end].isLetterOrNumber() && text[end] != '_')
+		    break;
+	    }
+	    state = 0;
+	    if (std::binary_search(ckw, ckw + sizeof(ckw)/sizeof(ckw[0]),
+			text.mid(start, end-start)))
+	    {
+		setFormat(start, end-start, identFont);
+	    } else {
+		setFormat(start, end-start, m_srcWnd->colorGroup().text());
+	    }
+	    break;
+	default:
+	    for (end = start; end < int(text.length()); end++)
+	    {
+		if (text[end] == '/')
+		{
+		    if (end+1 < int(text.length())) {
+			if (text[end+1] == '/') {
+			    state = hlCommentLine;
+			    break;
+			} else if (text[end+1] == '*') {
+			    state = hlCommentBlock;
+			    break;
+			}
+		    }
+		}
+		else if (text[end] == '"' || text[end] == '\'')
+		{
+		    state = hlString;
+		    break;
+		}
+		else if (text[end] >= 'A' && text[end] <= 'Z' ||
+			 text[end] >= 'a' && text[end] <= 'z' ||
+			 text[end] == '_')
+		{
+		    state = hlIdent;
+		    break;
+		}
+	    }
+	    setFormat(start, end-start, m_srcWnd->colorGroup().text());
+	}
+	start = end;
+    }
+    return state;
+}
 
 #include "sourcewnd.moc"
