@@ -7,14 +7,16 @@
 #define SOURCEWND_H
 
 #include <qpixmap.h>
-#include "textvw.h"
+#include <qtextedit.h>
+#include <qsyntaxhighlighter.h>
+#include <vector>
 #include "dbgdriver.h"
 
 // forward declarations
 class KDebugger;
 struct DbgAddr;
 
-class SourceWindow : public KTextView
+class SourceWindow : public QTextEdit
 {
     Q_OBJECT
 public:
@@ -50,13 +52,12 @@ public:
     void activeLine(int& lineNo, DbgAddr& address);
 
 protected:
-    virtual int textCol() const;
-    virtual int cellWidth(int col) const;
-    virtual void paintCell(QPainter* p, int row, int col);
+    virtual void drawFrame(QPainter* p);
+    virtual bool eventFilter(QObject* watched, QEvent* e);
+    virtual void contextMenuEvent(QContextMenuEvent* e);
     virtual void mousePressEvent(QMouseEvent* ev);
     virtual void keyPressEvent(QKeyEvent* ev);
     virtual void paletteChange(const QPalette&);
-    void updateLineItem(int i);
     void expandRow(int row);
     void collapseRow(int row);
     void scrollToRow(int row);
@@ -69,10 +70,13 @@ protected:
 signals:
     void clickedLeft(const QString&, int, const DbgAddr& address, bool);
     void clickedMid(const QString&, int, const DbgAddr& address);
-    void clickedRight(const QPoint &);
     void disassemble(const QString&, int);
     void expanded(int lineNo);		/* source lineNo has been expanded */
     void collapsed(int lineNo);		/* source lineNo has been collapsed */
+    void lineChanged();
+public slots:
+    void setTabWidth(int numChars);
+    void cursorChanged(int row);
 
 protected:
     QString m_fileName;
@@ -84,16 +88,16 @@ protected:
 
     struct SourceLine {
 	QString code;			/* a line of text */
-	ValArray<QString> disass;	/* its disassembled code */
-	ValArray<DbgAddr> disassAddr;	/* the addresses thereof */
+	std::vector<QString> disass;		/* its disassembled code */
+	std::vector<DbgAddr> disassAddr;	/* the addresses thereof */
 	bool canDisass;			/* if line can be disassembled */
 	SourceLine() : canDisass(true) { }
 	int findAddressRowOffset(const DbgAddr& address) const;
     };
-    ValArray<SourceLine> m_sourceCode;
+    std::vector<SourceLine> m_sourceCode;
 
-    ValArray<int> m_rowToLine;		/* source line number for each row */
-    ValArray<uchar> m_lineItems;
+    std::vector<int> m_rowToLine;	//!< The source line number for each row
+    std::vector<uchar> m_lineItems;	//!< Icons displayed on the line
     QPixmap m_pcinner;			/* PC at innermost frame */
     QPixmap m_pcup;			/* PC at frame up the stack */
     QPixmap m_brkena;			/* enabled breakpoint */
@@ -101,6 +105,18 @@ protected:
     QPixmap m_brktmp;			/* temporary breakpoint marker */
     QPixmap m_brkcond;			/* conditional breakpoint marker */
     QPixmap m_brkorph;			/* orphaned breakpoint marker */
+    int m_curRow;			//!< The highlighted row
+    int m_widthItems;			//!< The width of the item column
+    int m_widthPlus;			//!< The width of the expander column
+};
+
+class HighlightCpp : public QSyntaxHighlighter
+{
+    SourceWindow* m_srcWnd;
+
+public:
+    HighlightCpp(SourceWindow* srcWnd);
+    virtual int highlightParagraph(const QString& text, int state);
 };
 
 #endif // SOURCEWND_H
