@@ -126,8 +126,6 @@ void STTY::outReceived(int f)
 	if (n < 0) {
 	    if (errno != EAGAIN) {	/* this is not an error */
 		// ugh! error! somebody disconnect this signal please!
-		int e = errno;
-		TRACE(QString().sprintf("error reading tty: %d", e));
 	    }
 	    break;
 	}
@@ -140,16 +138,15 @@ void STTY::outReceived(int f)
 
 
 TTYWindow::TTYWindow(QWidget* parent, const char* name) :
-	KTextView(parent, name),
-	m_tty(0)
+	QTextEdit(parent, name),
+	m_tty(0),
+	m_hPos(0)
 {
     setFont(KGlobalSettings::fixedFont());
-    setFocusPolicy(WheelFocus);
-    clear();
-
-    // create a context menu
-    m_popmenu = new QPopupMenu;
-    m_popmenu->insertItem(i18n("&Clear"), this, SLOT(clear()));
+    setReadOnly(true);
+    setAutoFormatting(AutoNone);
+    setTextFormat(PlainText);
+    setWordWrap(NoWrap);
 }
 
 TTYWindow::~TTYWindow()
@@ -182,59 +179,60 @@ void TTYWindow::deactivate()
     m_tty = 0;
 }
 
+/**
+ * Note that it is necessary to track the horizontal position explicitly
+ * since if the user modifies the selected text in the window, the cursor
+ * position changes, too.
+ */
 void TTYWindow::slotAppend(char* buffer, int count)
 {
-    // is last line visible?
-    bool bottomVisible = lastRowVisible() == m_texts.size()-1;
-
     // parse off lines
     char* start = buffer;
     while (count > 0) {
 	int len = 0;
-	while (count > 0 && start[len] != '\n') {
+	while (count > 0 && start[len] != '\n' && start[len] != '\r') {
 	    --count;
 	    ++len;
 	}
 	if (len > 0) {
 	    QString str = QString::fromLatin1(start, len);
-	    // update last line
-	    str = m_texts[m_texts.size()-1] + str;
-	    replaceLine(m_texts.size()-1, str);
+	    // replace text in the last line
+	    int para = paragraphs()-1;
+	    // this selection is non-empty only after a '\r' that was not
+	    // followed by a '\n'
+	    setSelection(para, m_hPos, para, m_hPos+len, 1);
+	    removeSelectedText(1);
+	    insertAt(str, para, m_hPos);
+	    m_hPos += len;
 	    start += len;
 	    len = 0;
 	}
-	if (count > 0 && *start == '\n') {
-	    insertLine(QString());
+	if (count > 0 && *start == '\r') {
 	    ++start;
 	    --count;
+	    m_hPos = 0;
+	}
+	if (count > 0 && *start == '\n') {
+	    ++start;
+	    --count;
+	    append(QString());
+	    m_hPos = 0;
 	}
     }
-
-    // if last row was visible, scroll down to make it visible again
-    if (bottomVisible)
-	setTopCell(m_texts.size()-1);
 }
 
-void TTYWindow::clear()
+QPopupMenu* TTYWindow::createPopupMenu(const QPoint& pos)
 {
-    m_texts.setSize(0);
-    m_width = 300; m_height = 14;	/* Same as in KTextView::KTextView */
-    insertLine(QString());
+    QPopupMenu* menu = QTextEdit::createPopupMenu(pos);
+    menu->insertSeparator();
+    menu->insertItem(i18n("&Clear"), this, SLOT(slotClear()));
+    return menu;
 }
 
-void TTYWindow::mousePressEvent(QMouseEvent* mouseEvent)
+void TTYWindow::slotClear()
 {
-    // Check if right button was clicked.
-    if (mouseEvent->button() == RightButton)
-    {
-	if (m_popmenu->isVisible()) {
-	    m_popmenu->hide();
-	} else {
-	    m_popmenu->popup(mapToGlobal(mouseEvent->pos()));
-	}
-    } else {
-	QWidget::mousePressEvent(mouseEvent);
-    }
+    clear();
+    m_hPos = 0;
 }
 
 #include "ttywnd.moc"
