@@ -54,6 +54,7 @@ class ValueTip : public QToolTip
 {
 public:
     ValueTip(WinStack* parent);
+    virtual ~ValueTip() {}	// Qt3's QToolTip lacks virtual dtor!
     virtual void maybeTip(const QPoint& p);
     void tip(const QRect& r, const QString& s) { QToolTip::tip(r, s); }
 };
@@ -95,8 +96,6 @@ signals:
     void lineChanged();
     void toggleBreak(const QString&, int, const DbgAddr&, bool);
     void enadisBreak(const QString&, int, const DbgAddr&);
-    void clickedRight(const QPoint&);
-    void filesRightClick(const QPoint&);
     void newFileLoaded();
     void initiateValuePopup(const QString&);
     void disassemble(const QString&, int);
@@ -134,7 +133,7 @@ protected:
     bool activatePath(QString pathname, int lineNo, const DbgAddr& address);
     virtual bool activateWindow(SourceWindow* fw, int lineNo, const DbgAddr& address);	/* -1 doesnt change line */
     virtual void changeWindowMenu();
-    virtual void mousePressEvent(QMouseEvent*);
+    virtual void contextMenuEvent(QContextMenuEvent* e);
     void setPC(bool set, const QString& fileName, int lineNo,
 	       const DbgAddr& address, int frameNo);
     ValArray<SourceWindow*> m_fileList;
