@@ -19,10 +19,10 @@
 #include "mydebug.h"
 
 
-static VarTree *parseVar(const char *&s);
+static ExprValue *parseVar(const char *&s);
 static bool parseName(const char *&s, QString & name,
                       VarTree::NameKind & kind);
-static bool parseValue(const char *&s, VarTree * variable);
+static bool parseValue(const char *&s, ExprValue * variable);
 static bool isErrorExpr(const char *output);
 
 #define TERM_IO_ALLOWED 1
@@ -409,7 +409,7 @@ XsldbgDriver::makeCmdString(DbgCommand cmd, QString strArg)
 	m_xslFile = strArg;
     }
 
-    SIZED_QString(cmdString, MAX_FMTLEN + strArg.length());
+    QString cmdString;
     cmdString.sprintf(cmds[cmd].fmt, strArg.latin1());
     return cmdString;
 }
@@ -420,8 +420,7 @@ XsldbgDriver::makeCmdString(DbgCommand cmd, int intArg)
     assert(cmd >= 0 && cmd < NUM_CMDS);
     assert(cmds[cmd].argsNeeded == XsldbgCmdInfo::argNum);
 
-    SIZED_QString(cmdString, MAX_FMTLEN + 30);
-
+    QString cmdString;
     cmdString.sprintf(cmds[cmd].fmt, intArg);
     return cmdString;
 }
@@ -436,7 +435,7 @@ XsldbgDriver::makeCmdString(DbgCommand cmd, QString strArg, int intArg)
 
     normalizeStringArg(strArg);
 
-    SIZED_QString(cmdString, MAX_FMTLEN + 30 + strArg.length());
+    QString cmdString;
 
     if (cmd == DCtty) {
         /*
@@ -535,8 +534,7 @@ XsldbgDriver::makeCmdString(DbgCommand cmd, QString strArg1,
     normalizeStringArg(strArg1);
     normalizeStringArg(strArg2);
 
-    SIZED_QString(cmdString,
-                  MAX_FMTLEN + strArg1.length() + strArg2.length());
+    QString cmdString;
     cmdString.sprintf(cmds[cmd].fmt, strArg1.latin1(), strArg2.latin1());
     return cmdString;
 }
@@ -547,7 +545,7 @@ XsldbgDriver::makeCmdString(DbgCommand cmd, int intArg1, int intArg2)
     assert(cmd >= 0 && cmd < NUM_CMDS);
     assert(cmds[cmd].argsNeeded == XsldbgCmdInfo::argNum2);
 
-    SIZED_QString(cmdString, MAX_FMTLEN + 60);
+    QString cmdString;
     cmdString.sprintf(cmds[cmd].fmt, intArg1, intArg2);
     return cmdString;
 }
@@ -700,21 +698,21 @@ isErrorExpr(const char *output)
 
 /**
  * Returns true if the output is an error message. If wantErrorValue is
- * true, a new VarTree object is created and filled with the error message.
+ * true, a new ExprValue object is created and filled with the error message.
  */
 static bool
 parseErrorMessage(const char *output,
-                  VarTree * &variable, bool wantErrorValue)
+                  ExprValue * &variable, bool wantErrorValue)
 {
     if (isErrorExpr(output)) {
         if (wantErrorValue) {
             // put the error message as value in the variable
-            variable = new VarTree(QString(), VarTree::NKplain);
+            variable = new ExprValue(QString(), VarTree::NKplain);
             const char *endMsg = strchr(output, '\n');
 
             if (endMsg == 0)
                 endMsg = output + strlen(output);
-            variable->m_value = FROM_LATIN1(output, endMsg - output);
+            variable->m_value = QString::fromLatin1(output, endMsg - output);
         } else {
             variable = 0;
         }
@@ -729,22 +727,22 @@ XsldbgDriver::setPrintQStringDataCmd(const char* /*cmd*/)
 {
 }
 
-VarTree *
+ExprValue *
 XsldbgDriver::parseQCharArray(const char */*output*/, bool /*wantErrorValue*/,
                               bool /*qt3like*/)
 {
-    VarTree *variable = 0;
+    ExprValue *variable = 0;
 
     TRACE("XsldbgDriver::parseQCharArray not implmented");
     return variable;
 }
 
-static VarTree *
+static ExprValue *
 parseVar(const char *&s)
 {
     const char *p = s;
     bool foundLocalVar = false;
-    VarTree *variable = 0L;
+    ExprValue *variable = 0L;
     QString name;
 
     VarTree::NameKind kind;
@@ -774,12 +772,11 @@ parseVar(const char *&s)
 	TRACE("Found print expr");
         if (nextLine) {
             p = p + 2;          /* skip the "= " */
-            name = FROM_LATIN1(p, nextLine - p);
+            name = QString::fromLatin1(p, nextLine - p);
             kind = VarTree::NKplain;
             p = nextLine + 1;
-            variable = new VarTree(name, kind);
+            variable = new ExprValue(name, kind);
             variable->m_varKind = VarTree::VKsimple;
-            variable->setDeleteChildren(true);
             parseValue(p, variable);
             return variable;
         }
@@ -796,10 +793,9 @@ parseVar(const char *&s)
       if (!parseName(p, name, kind)) {
         return 0;
       }
-      variable = new VarTree(name, kind);
+      variable = new ExprValue(name, kind);
       if (variable != 0L) {
 	  variable->m_varKind = VarTree::VKsimple;
-        variable->setDeleteChildren(true);
       }
     }else{
       p++;
@@ -810,10 +806,9 @@ parseVar(const char *&s)
       if (!parseName(p, name, kind)) {
         return 0;
       }
-      variable = new VarTree(name, kind);
+      variable = new ExprValue(name, kind);
       if (variable != 0L) {
 	  variable->m_varKind = VarTree::VKsimple;
-	  variable->setDeleteChildren(true);
       }
       if (*p == '\n')
 	p++;
@@ -857,7 +852,7 @@ parseName(const char *&s, QString & name, VarTree::NameKind & kind)
     }
 
 
-    name = FROM_LATIN1(s, len);
+    name = QString::fromLatin1(s, len);
     /* XSL variables will have a $ prefix to be evaluated 
      * properly */
     //TRACE(QString("parseName got name" ) +  name);
@@ -868,10 +863,10 @@ parseName(const char *&s, QString & name, VarTree::NameKind & kind)
 }
 
 static bool
-parseValue(const char *&s, VarTree * variable)
+parseValue(const char *&s, ExprValue * variable)
 {
     const char *start = s, *end = s;
-    VarTree * childValue; 
+    ExprValue * childValue; 
     #define VALUE_END_MARKER_INDEX  0
 
     /* This mark the end of a value */
@@ -924,16 +919,16 @@ parseValue(const char *&s, VarTree * variable)
 	if ((variable->m_varKind == VarTree::VKsimple)) {
 	    if (!variable->m_value.isEmpty()){
 		variable->m_varKind = VarTree::VKarray;
-		childValue = new VarTree(variable->m_value, VarTree::NKplain);
+		childValue = new ExprValue(variable->m_value, VarTree::NKplain);
 		variable->appendChild(childValue);
-		childValue = new VarTree(valueBuffer, VarTree::NKplain);
+		childValue = new ExprValue(valueBuffer, VarTree::NKplain);
 		variable->appendChild(childValue);
 		variable->m_value = "";
 	    }else{
 		variable->m_value = valueBuffer;
 	    }
 	}else{
-	    childValue = new VarTree(valueBuffer, VarTree::NKplain);
+	    childValue = new ExprValue(valueBuffer, VarTree::NKplain);
 	    variable->appendChild(childValue);
 	}
 
@@ -1107,7 +1102,7 @@ XsldbgDriver::parseBackTrace(const char *output,
         frm->fileName = file;
         frm->lineNo = lineNo;
         frm->address = address;
-        frm->var = new VarTree(func, VarTree::NKplain);
+        frm->var = new ExprValue(func, VarTree::NKplain);
         stack.append(frm);
     }
 }
@@ -1313,19 +1308,19 @@ XsldbgDriver::parseBreakpoint(const char *output, int &id,
 }
 
 void
-XsldbgDriver::parseLocals(const char *output, QList < VarTree > &newVars)
+XsldbgDriver::parseLocals(const char *output, QList < ExprValue > &newVars)
 {
 
     /* keep going until error or xsldbg prompt is found */
     while (*output != '\0') {
-        VarTree *variable = parseVar(output);
+        ExprValue *variable = parseVar(output);
 
         if (variable == 0) {
             break;
         }
         // do not add duplicates
-        for (VarTree * o = newVars.first(); o != 0; o = newVars.next()) {
-            if (o->getText() == variable->getText()) {
+        for (ExprValue * o = newVars.first(); o != 0; o = newVars.next()) {
+            if (o->m_name == variable->m_name) {
                 delete variable;
 
                 goto skipDuplicate;
@@ -1337,18 +1332,16 @@ XsldbgDriver::parseLocals(const char *output, QList < VarTree > &newVars)
 }
 
 
-bool
-XsldbgDriver::parsePrintExpr(const char *output, bool wantErrorValue,
-                             VarTree * &var)
+ExprValue *
+XsldbgDriver::parsePrintExpr(const char *output, bool wantErrorValue)
 {
+    ExprValue* var = 0;
     // check for error conditions
-    if (parseErrorMessage(output, var, wantErrorValue)) {
-        return false;
-    } else {
+    if (!parseErrorMessage(output, var, wantErrorValue)) {
         // parse the variable
         var = parseVar(output);
-        return true;
     }
+    return var;
 }
 
 bool
